C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE DOSELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates <d|O|s> offdiagonal elements of the     *
C *               transition matrix. The operator operates to the    *
C *               left function.                                     *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: August 1992                                        *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             * 
C *                                                                  *
C ********************************************************************

      SUBROUTINE DOSELE (M, HQZ, N2, SK11, SK12, SK2, C1, C2,
     .                   R, PTR, DTR, FTR, LDIST, POL, CURRI)

      IMPLICIT NONE

        DOUBLE PRECISION M
        INTEGER          HQZ, N2
        DOUBLE PRECISION SK11, SK12, SK2, C1, C2, R, PTR(9)
        DOUBLE PRECISION DTR(25), FTR(49), LDIST
        CHARACTER*1      POL
        INTEGER          CURRI

C CONST
      INCLUDE 'dconst.inc'

C VAR
      DOUBLE PRECISION  A(20), B(20), A1(20), B1(20)
      DOUBLE PRECISION  DSOS1, DSOS2, DSO1(5), DSO2(5), FSOS1, FSOS2
      DOUBLE PRECISION  PSOS1, PSOS2, PSO1(3), PSO2(3), FSO1(7), FSO2(7)
      INTEGER            L1, L2, ML, MAX, N1
      DOUBLE PRECISION  NORM1, NORM2

      INCLUDE 'dfns.inc'
      INCLUDE 'dnorm.inc'

C BEGIN

C Berechne zuerst den "normalen" d-s-Overlap

      N1 = HQZ

      L1 = 2
      L2 = 0
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK11, SK2, R, MAX)
      CALL LOVLAP(DSOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL ABFNS(A1, B1, SK12, SK2, R, MAX)
      CALL LOVLAP(DSOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)
    
      CALL SDOVL(DSO1, DSOS1, DTR)
      CALL SDOVL(DSO2, DSOS2, DTR)

C Berechne nun den p-s-Overlap nachdem man den Operator auf das
C d-Orbital angewendet hat.

      N1 = N1 + 1
      L1 = 1
      MAX = N1 + N2

      CALL ABFNS(A, B, SK11, SK2, R, MAX)
      CALL LOVLAP(PSOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL ABFNS(A1, B1, SK12, SK2, R, MAX)
      CALL LOVLAP(PSOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      PSOS1 = -PSOS1
      PSOS2 = -PSOS2

      CALL SPOVL(PSO1, PSOS1, PTR)
      CALL SPOVL(PSO2, PSOS2, PTR)

C Berechne den f-s-Overlap, nachdem man den Operator auf das
C d-Orbital angewendet hat.

      L1 = 3

      CALL LOVLAP(FSOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FSOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      FSOS1 = -FSOS1
      FSOS2 = -FSOS2

      CALL SFOVL(FSO1, FSOS1, FTR)
      CALL SFOVL(FSO2, FSOS2, FTR)

      IF (CURRI .EQ. 5) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I5X(PSO1(1), DSO1(1), FSO1(6), FSO1(2), SK11) +
     .             C2 * I5X(PSO2(1), DSO2(1), FSO2(6), FSO2(2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I5Y(PSO1(2), DSO1(1), FSO1(7), FSO1(3), SK11) +
     .             C2 * I5Y(PSO2(2), DSO2(1), FSO2(7), FSO2(3), SK12))
        ELSE
          M = M * (C1 * XYZ(DSO1(1), FSO1(5), SK11) +
     .             C2 * XYZ(DSO2(1), FSO2(5), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 6) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I6XY(PSO1(1), DSO1(2), FSO1(2), SK11) +
     .             C2 * I6XY(PSO2(1), DSO2(2), FSO2(2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I6XY(PSO1(2), DSO1(2), FSO1(3), SK11) +
     .             C2 * I6XY(PSO2(2), DSO2(2), FSO2(3), SK12))
        ELSE
          M = M * (C1 * I6Z(PSO1(3), DSO1(2), FSO1(1), SK11) +
     .             C2 * I6Z(PSO2(3), DSO2(2), FSO2(1), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 7) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I7X(PSO1(2), DSO1(3), FSO1(7), FSO1(3), SK11) +
     .             C2 * I7X(PSO2(2), DSO2(3), FSO2(7), FSO2(3), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I7Y(PSO1(1), DSO1(3), FSO1(6), FSO1(2), SK11) +
     .             C2 * I7Y(PSO2(1), DSO2(3), FSO2(6), FSO2(2), SK12))
        ELSE
          M = M * (C1 * XYZ(DSO1(3), FSO1(4), SK11) +
     .             C2 * XYZ(DSO2(3), FSO2(4), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 8) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I8X(PSO1(3), DSO1(4), FSO1(5), FSO1(1), SK11) +
     .             C2 * I8X(PSO2(3), DSO2(4), FSO2(5), FSO2(1), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * XYZ(DSO1(4), FSO1(4), SK11) +
     .             C2 * XYZ(DSO2(4), FSO2(4), SK12))
        ELSE
          M = M * (C1 * I8Z(PSO1(1), DSO1(4), FSO1(2), SK11) +
     .             C2 * I8Z(PSO2(1), DSO2(4), FSO2(2), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 9) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * XYZ(DSO1(5), FSO1(4), SK11) +
     .             C2 * XYZ(DSO2(5), FSO2(4), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I9Y(PSO1(3), DSO1(5), FSO1(5), FSO1(1), SK11) +
     .             C2 * I9Y(PSO2(3), DSO2(5), FSO2(5), FSO2(1), SK12))
        ELSE
          M = M * (C1 * I9Z(PSO1(2), DSO1(5), FSO1(3), SK11) +
     .             C2 * I9Z(PSO2(2), DSO2(5), FSO2(3), SK12))
        ENDIF

      ENDIF


      RETURN
      END

C END
