C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE EULERP                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the Euler-matrix for the projection of  *
C *               the p-orbitals.                                    *
C *                                                                  *
C *  Authors....: unknown                                            *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: July 1992 (Ruedi Rytz)                             *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE EULERP(X, Y, Z, LATOM, HATOM, R,
     .                  CB, SB, SA, CA, PTR, OFFSET)

      IMPLICIT NONE

      INCLUDE 'sizes'


        DOUBLE PRECISION X(MAXATO), Y(MAXATO), Z(MAXATO)
        INTEGER          LATOM, HATOM
        DOUBLE PRECISION R, CB, SB, SA, CA, PTR(9), OFFSET(3)

C CONST 
      DOUBLE PRECISION AUI
      PARAMETER (AUI   = 1.889727D0)

C VAR
      DOUBLE PRECISION   DELX, DELY, DELZ, RT2, T

C BEGIN

      DELX = X(LATOM) - ( X(HATOM) + OFFSET(1) )
      DELY = Y(LATOM) - ( Y(HATOM) + OFFSET(2) )
      DELZ = Z(LATOM) - ( Z(HATOM) + OFFSET(3) )
      RT2  = DELX*DELX + DELY*DELY
      R = SQRT( RT2 + DELZ*DELZ )

      IF (RT2 .GT. 1.0D-10) THEN
        T = SQRT(RT2)
        CB = DELX/T
        SB = DELY/T
        SA = T/R
        CA = DELZ/R
      ELSE
        CB = 1.0D0
        SB = 0.0D0
        SA = 0.0D0
        CA = DELZ/R
      ENDIF

      PTR(1)= SA*CB
      PTR(2)= SA*SB
      PTR(3)= CA
      PTR(4)= CA*CB
      PTR(5)= CA*SB
      PTR(6)= -SA
      PTR(7)= -SB
      PTR(8)= CB
      PTR(9)= 0.0D0

      R = R * AUI

      RETURN
      END

C END
