C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE MATRIX                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       *
C *                                                                  *
C *  Usage......: Calculates the transition matrix.                  *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: June 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: READIN                                             *
C *                                                                  *
C *  Calls......: DIAG                                               *
C *               BDIAG                                              *
C *               OBDIAG                                             *
C *               OUT                                                *
C *               MOUT                                               *
C *                                                                  *
C ********************************************************************

      SUBROUTINE MATRIX(PSII, CPSII, PSIF, CPSIF, ANZAO, ANZATO,
     .                  X, Y, Z, FAO, LAO, POL, NS, NP, ND,
     .                  POINTER, EXPS, EXPP, EXPD1, EXPD2,
     .                  C1, C2, ZDO, EPSII, EPSIF, NPSII, NPSIF,
     .                  KEYWRD, DIM, NEIGH, KP, WEIGHT)

      IMPLICIT NONE

      INCLUDE 'sizes'
      INCLUDE 'edconst.inc'

        DOUBLE PRECISION  PSII(MAXAO), CPSII(MAXAO), PSIF(MAXAO)
        DOUBLE PRECISION  CPSIF(MAXAO)
        INTEGER           ANZAO, ANZATO
        DOUBLE PRECISION  X(MAXATO), Y(MAXATO), Z(MAXATO)
        INTEGER           FAO(MAXATO), LAO(MAXATO)
        CHARACTER*1       POL
        INTEGER           NS(MAXATO), NP(MAXATO), ND(MAXATO) 
        INTEGER           POINTER(MAXATO)
        DOUBLE PRECISION  EXPS(MAXATO), EXPP(MAXATO), EXPD1(MAXATO)
        DOUBLE PRECISION  EXPD2(MAXATO), C1(MAXATO), C2(MAXATO)
        LOGICAL           ZDO
        DOUBLE PRECISION  EPSII, EPSIF
        INTEGER           NPSII, NPSIF
        CHARACTER*(4*RECLEN) KEYWRD
        DOUBLE PRECISION  DIM(3,3) 
        INTEGER           NEIGH(3)
        DOUBLE PRECISION  KP(3), WEIGHT

C VAR
      COMMON /TRANS/WAVNUM(MAXTRA), OSCX(MAXTRA),
     .              OSCY(MAXTRA), OSCZ(MAXTRA), DEGEN(MAXAO),
     .              COUNTER, INMO(MAXTRA), FIMO(MAXTRA)
      DOUBLE PRECISION WAVNUM, OSCX, OSCY, OSCZ
      INTEGER DEGEN, COUNTER
      INTEGER INMO, FIMO

      COMMON /IOUNIT/IIN, IOUT, ISTART(40)
      INTEGER IIN, IOUT, ISTART

      INTEGER I, J, N, DIV  
      DOUBLE PRECISION M(MAXAO,MAXAO), SUM, DELTAE
      DOUBLE PRECISION OFFSET(3)

      LOGICAL SAME

C BEGIN
      IF (INDEX(KEYWRD,'CRYST') .GT. 0) THEN

        CALL CRYST(SUM,PSII,CPSII,PSIF,CPSIF,ANZAO,KP,DIM,NEIGH,ANZATO,
     .             X,Y,Z,FAO,LAO,POL,NS,NP,ND,POINTER,EXPS,EXPP,EXPD1, 
     .             EXPD2,C1,C2)

      ELSE

        DO 10 I = 1, ANZAO
          DO 20 J = 1, ANZAO
            M(I,J) = PSII(I) * PSIF(J)
20        CONTINUE
10      CONTINUE

        CALL DIAG (M, ANZATO,
     .             X, Y, Z, FAO, LAO, POL)

        CALL BDIAG(M, ANZATO, FAO, LAO, POL, NS, NP,
     .             POINTER, EXPS, EXPP, EXPD1, EXPD2,
     .             C1, C2)

        IF (ZDO) THEN
          N = 1
          SUM = 0.0D0

C Schlaufe ueber die obere rechte Haelfte der Matrix (inclusive der
C Diagonalen)

          DO 30 I = 1, ANZAO
            DO 40 J = I, ANZAO

C IF (I. LE. LAO(N) .AND. J .LE. LAO(N)) THEN... betrachte nur den Block-
C diagonalen Teil der Matrix.

              IF (I .LE. LAO(N) .AND. J .LE. LAO(N)) THEN
                IF (I .EQ. J) THEN
                  SUM = SUM + M(I,J)
                ELSE
                  SUM = SUM + M(I,J) + M(J,I)
                ENDIF

C IF (I .EQ. LAO(N) .AND. J .EQ. ANZAO) THEN... wenn man zum letzten
C Atomorbital am selben Atom gekommen ist und man bei der letzten Kolonne
C der Matrix ist, so gehe zum naechsten Atom ueber.

              ELSE
                IF (I .EQ. LAO(N) .AND. J .EQ. ANZAO) THEN
                  N = N + 1
                ENDIF
                M(I,J) = 0.0
                M(J,I) = 0.0
              ENDIF

40          CONTINUE
30        CONTINUE

        ELSE

          SAME=.TRUE.
          DO 70 I = 1, 3
            OFFSET(I)=0.0D0
70        CONTINUE
          CALL OBDIAG(M, ANZATO, FAO, LAO, POL, NS, NP, ND,
     .                POINTER, EXPS, EXPP, EXPD1, EXPD2,
     .                C1, C2, X, Y, Z, SAME, OFFSET)

          SUM = 0.0D0
          DO 50 I = 1,ANZAO
            DO 60 J = I, ANZAO
              IF (I .EQ. J) THEN
                SUM = SUM + M(I,J)
              ELSE
                SUM = SUM + M(I,J) + M(J,I)
              ENDIF
60          CONTINUE
50        CONTINUE

        ENDIF 
      ENDIF


      IF (INDEX(KEYWRD,'SPECOUT') .GT. 0) THEN

        WAVNUM(COUNTER) = DELTAE(EPSII, EPSIF) * CM  
        IF (POL .EQ. 'X') THEN
          OSCX(COUNTER) = ( FAC * SUM**2 * WAVNUM(COUNTER) * WEIGHT) /
     .                     DEGEN(NPSII)
        ELSEIF (POL .EQ. 'Y') THEN 
          OSCY(COUNTER) = ( FAC * SUM**2 * WAVNUM(COUNTER) * WEIGHT) /
     .                     DEGEN(NPSII)
        ELSE
          OSCZ(COUNTER) = ( FAC * SUM**2 * WAVNUM(COUNTER) * WEIGHT) / 
     .                     DEGEN(NPSII)
        ENDIF
        INMO(COUNTER) = NPSII
        FIMO(COUNTER) = NPSIF
      ENDIF

      IF (INDEX(KEYWRD,'TRANSOUT') .GT. 0) THEN

        IF (INDEX(KEYWRD,'CORRECT') .EQ. 0) THEN
          DIV = 1 
        ELSE
          DIV = DEGEN(NPSII)
        ENDIF
        CALL OUT(SUM, POL, DIV, EPSII, EPSIF)

      ENDIF
   
      IF (INDEX(KEYWRD,'MATOUT') .GT. 0) THEN

        WRITE(IOUT, '(//, A, A1, A, /)' ) 'The transition matrix for '
     .              ,POL,' polarisation:'
        CALL MOUT (M,ANZAO,10)

      ENDIF

      RETURN
      END
C END
