C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE OBDIAG                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       *
C *                                                                  *
C *  Usage......: Calculates the off-blockdiagonal elements of       *
C *               the transition matrix.                             *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: July 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: MATRIX                                             *
C *                                                                  *
C *  Calls......: EULERP                                             *
C *               EULERD                                             *
C *               EULERF                                             *
C *               SOSELE                                             *
C *               SOPELE                                             *
C *               SODELE                                             *
C *               POSELE                                             *
C *               POPELE                                             *
C *               PODELE                                             *
C *               DOSELE                                             *
C *               DOPELE                                             *
C *               DODELE                                             *
C *                                                                  *
C ********************************************************************

      SUBROUTINE  OBDIAG(M, ANZATO, FAO, LAO, POL, NS, NP, ND,
     .                   POINTER, EXPS, EXPP, EXPD1, EXPD2,
     .                   C1, C2, X, Y, Z, SAME, OFFSET)

      IMPLICIT NONE

      INCLUDE 'sizes'

        DOUBLE PRECISION  M(MAXAO,MAXAO)
        INTEGER           ANZATO
        INTEGER           FAO(MAXATO), LAO(MAXATO)
        CHARACTER*1       POL
        INTEGER           NS(MAXATO), NP(MAXATO), ND(MAXATO)
        INTEGER           POINTER(MAXATO)
        DOUBLE PRECISION  EXPS(MAXATO), EXPP(MAXATO), EXPD1(MAXATO)
        DOUBLE PRECISION  EXPD2(MAXATO), C1(MAXATO), C2(MAXATO)
        DOUBLE PRECISION  X(MAXATO), Y(MAXATO), Z(MAXATO)
        LOGICAL           SAME
        DOUBLE PRECISION  OFFSET(3)

C CONST
      DOUBLE PRECISION AUI
      PARAMETER (AUI   = 1.889727D0)

C VAR
      INTEGER  I, J, N, O
      INTEGER  CURRI, CURRJ
      DOUBLE PRECISION  LDIST
      DOUBLE PRECISION  R, SA, SB, CA, CB, PTR(9), DTR(25), FTR(49)
      LOGICAL  DOIT, DOIT1, DOIT2

C BEGIN

C In CURRJ (current j) und CURRI stehen die Informationen um welche Typen von
C Orbitalen es sich handelt. CURRI ist am N-ten Atom zentriert.
C z. B. CURRI = 1 bedeutet ein s-Orbital, CURRI = 4 ist ein pz-Orbital.

      DOIT=.FALSE.

      DO 10 N = 1, ANZATO
        DO 20 I = FAO(N), LAO(N)
          CURRI = I - FAO(N) + 1
          DO 30 O = 1, ANZATO
            DO 40 J = FAO(O), LAO(O)
              CURRJ = J - FAO(O) + 1
            
              DOIT1 = ( SAME .AND. (N .NE. O) .AND.
     .                  (ABS(M(I,J)) .GT. 1.0D-10) )
              DOIT2 = ( .NOT. SAME .AND.
     .                  (ABS(M(I,J)) .GT. 1.0D-10) )
              DOIT  = DOIT1 .OR. DOIT2
              IF (DOIT) THEN

C In LDIST speichern wir die Distanz in Polarisationsrichtung vom
C Zentrum des Koordintensystems zum n-ten Atom, auf das der Operator
C wirkt.
                IF (POL .EQ. 'X') THEN
                  LDIST = X(N) * AUI
                ELSEIF (POL .EQ. 'Y') THEN
                  LDIST = Y(N) * AUI
                ELSE
                  LDIST = Z(N) * AUI
                ENDIF

C Vorgehen: 1) Berechnung des Abstands der beiden Atome
C           2) Berechnung der Eulertransformationsmatrix
C           3) Berechnung der winkelunabhaengigen Ueberlappung
C           4) Transformation mit der Eulermatrix
C           5) Neue Normierung (fuer contracted orbitals)

C EULERP berechnet die Eulermatrix fuer s-p und p-p Ueberlappungen

                CALL EULERP(X, Y, Z, N, O, R,
     .                      CB, SB, SA, CA, PTR, OFFSET)


C IF (CURRI.EQ. 1) THEN.. heisst wenn s-Orbital
C IF (CURRJ.EQ. 1) THEN.. heisst beides s-Orbitale
C ==> S(CURRI)^S(CURRJ)-Overlap

C*****************************************************************************
C Berechnung der <s|O|s> - Uebergangselemente                                *
C*****************************************************************************
                IF (CURRI .EQ. 1) THEN
                  IF (CURRJ .EQ. 1) THEN

                    CALL SOSELE(M(I,J), NS(POINTER(N)),
     .                          NS(POINTER(O)),  EXPS(POINTER(N)),
     .                          EXPS(POINTER(O)), R, PTR, LDIST, POL)

                  ELSEIF (CURRJ .GE. 2 .AND. CURRJ .LE. 4) THEN

C*****************************************************************************
C Berechnung der <s|O|p> - Uebergangselemente                                *
C*****************************************************************************

C PSOS bedeute p-s-sigma Overlap

C Berechnung der pp-Overlaps = erster Teil des gesamten Uebergangsintegral,
C das in 2 Teile zerfaellt.
C Wichtig: Die Operatoren wirken immmer nach links. Es wird der Uebergang
C <PSII | Operator | PSIF> betrachtet.

                    CALL SOPELE(M(I,J), NS(POINTER(N)),
     .                          NP(POINTER(O)),  EXPS(POINTER(N)),
     .                          EXPP(POINTER(O)), R, PTR, LDIST, POL,
     .                          CURRJ)

                  ELSEIF (CURRJ .GE. 5 .AND. CURRJ .LE. 9) THEN

C*****************************************************************************
C Berechnung der <s|O|d> - Uebergangselemente                                *
C*****************************************************************************
                    CALL EULERD(CB, SB, SA, CA, DTR)

                    CALL SODELE(M(I,J), NS(POINTER(N)),
     .                          ND(POINTER(O)), EXPS(POINTER(N)),
     .                          EXPD1(POINTER(O)), EXPD2(POINTER(O)),
     .                          C1(POINTER(O)), C2(POINTER(O)), R,
     .                          PTR, DTR, LDIST, POL, CURRJ)

                  ENDIF

                ELSEIF (CURRI .GE. 2 .AND. CURRI .LE. 4) THEN

C*****************************************************************************
C Berechnung der <p|O|s> - Uebergangselemente                                *
C*****************************************************************************

                  CALL EULERD(CB, SB, SA, CA, DTR)

                  IF (CURRJ .EQ. 1) THEN

                    CALL POSELE(M(I,J), NP(POINTER(N)),
     .                          NS(POINTER(O)), EXPP(POINTER(N)),
     .                          EXPS(POINTER(O)), R, PTR,
     .                          DTR, LDIST, POL, CURRI)

                  ELSEIF (CURRJ .GE. 2 .AND. CURRJ .LE. 4) THEN

C*****************************************************************************
C Berechnung der <p|O|p> - Uebergangselemente                                *
C*****************************************************************************

                    CALL POPELE(M(I,J), NP(POINTER(N)),
     .                          NP(POINTER(O)), EXPP(POINTER(N)),
     .                          EXPP(POINTER(O)), R, PTR,
     .                          DTR, LDIST, POL, CURRI, CURRJ)

                  ELSEIF (CURRJ .GE. 5 .AND. CURRJ .LE. 9) THEN

C*****************************************************************************
C Berechnung der <p|O|d> - Uebergangselemente                                *
C*****************************************************************************

                    CALL PODELE(M(I,J), NP(POINTER(N)),
     .                          ND(POINTER(O)), EXPP(POINTER(N)),
     .                          EXPD1(POINTER(O)), EXPD2(POINTER(O)),
     .                          C1(POINTER(O)), C2(POINTER(O)),
     .                          R, PTR, DTR, LDIST, POL, CURRI, CURRJ)

                  ENDIF

                ELSEIF (CURRI .GE. 5 .AND. CURRI .LE. 9) THEN

C*****************************************************************************
C Berechnung der <d|O|s> - Uebergangselemente                                *
C*****************************************************************************

                  CALL EULERD(CB, SB, SA, CA, DTR)
                  CALL EULERF(CB, SB, SA, CA, FTR)

                  IF (CURRJ .EQ. 1) THEN

                    CALL DOSELE(M(I,J), ND(POINTER(N)),
     .                          NS(POINTER(O)), EXPD1(POINTER(N)),
     .                          EXPD2(POINTER(N)), EXPS(POINTER(O)),
     .                          C1(POINTER(N)), C2(POINTER(N)), R,
     .                          PTR, DTR, FTR, LDIST, POL, CURRI)

                  ELSEIF (CURRJ .GE. 2 .AND. CURRJ .LE. 4) THEN

C*****************************************************************************
C Berechnung der <d|O|p> - Uebergangselemente                                *
C*****************************************************************************

                    CALL DOPELE(M(I,J), ND(POINTER(N)),
     .                          NP(POINTER(O)), EXPD1(POINTER(N)),
     .                          EXPD2(POINTER(N)), EXPP(POINTER(O)),
     .                          C1(POINTER(N)), C2(POINTER(N)), R,
     .                          PTR, DTR, FTR, LDIST, POL, CURRI,
     .                          CURRJ)

                  ELSEIF (CURRJ .GE. 5 .AND. CURRJ .LE. 9) THEN

C*****************************************************************************
C Berechnung der <d|O|d> - Uebergangselement                                 *
C*****************************************************************************

                    CALL DODELE(M(I,J), ND(POINTER(N)), ND(POINTER(O)),
     .                          EXPD1(POINTER(N)), EXPD2(POINTER(N)),
     .                          EXPD1(POINTER(O)), EXPD2(POINTER(O)),
     .                          C1(POINTER(N)), C2(POINTER(N)),
     .                          C1(POINTER(O)), C2(POINTER(O)), R, PTR,
     .                          DTR, FTR, LDIST, POL, CURRI, CURRJ)

                  ENDIF

                ENDIF ! (CURRI .EQ. 1)

              ENDIF
40          CONTINUE
30        CONTINUE
20      CONTINUE
10    CONTINUE

      RETURN
      END
C END
