C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE EVQR                                                 *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Performs a QR transform on a real symmetric tri-   *
C *               diagonal matrix.                                   *
C *                                                                  *
C *               ON ENTERING, A CONTAINS THE N DIAGONAL ELEMENTS OF *
C *               THE TRIDIAGONAL MATRIX. B CONTAINS THE SQUARES OF  *
C *               THE N-1 OFF-DIAGONAL ELEMENTS. ITERATION IS CONTI- *
C *               NUED UNTIL THE SQUARES OF THE OFFDIAGONAL ELEMENTS *
C *               ARE LESS THAN TOL. TYPICALLY LESS THAN TWO ITERA-  *
C *               TIONS PER EIGENVALUE ARE REQUIRED. THUS THE UPPER  *
C *               LIMIT M TO THE NUMBER OF C ITERATIONS PER EIGENVA- *
C *               LUE MAY BE  SAFELY SET AT 20 OR SO.                *
C *                                                                  *
C *  Authors....: see subroutine GIVENS                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *  Date.......: -                                                  *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: GIVENS                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE EVQR(A,B,N,M,TOL)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION A(1),B(1)

      NX = N
      NI = 1
      SH = 0.D0
C
C  K COUNTS THE NUMBER OF ITERATIONS PER EIGENVALUE.
C
      K = 0
      IF (NX-2) 50,60,85
C
C  EACH NEW ITERATION BEGINS HERE.
C
100   K = K+1
      IF (K.LT.M) GOTO 101
      WRITE (6,1000) K
1000  FORMAT(35H NO CONVERGENCE OF QR ALGORITHM IN   ,I4,11H ITERATIONS)
      STOP
C
C  SOLVE THE TWO BY TWO IN THE LOWER RIGHT CORNER AND USE SMALLER ROOT
C  AS THE NEXT SHIFT.
C
101   AT = A(NX)+A(NX-1)
      ST = AT*5.D-1
      DISC = AT**2-4.D0*(A(NX)*A(NX-1)-B(NX-1))
      IF (DISC.LE.0.D0) GOTO 15
      ST = ST-DSIGN(DSQRT(DISC),ST)*5.D-1
C
C  INCREASE THE TOTAL SHIFT BY THE TEMPORARY SHIFT.
C
15    SH = SH+ST
C
C  THIS LOOP SUBTRACTS THE TEMPORARY SHIFT FROM THE DIAGONAL ELEMENTS.
C
      DO 20 I=1,NX
20      A(I) = A(I)-ST
C
C  INITIALIZE.
C
      G = A(NI)
      PS = G**2
      RS = PS+B(NI)
      SX = B(NI)/RS
      CXS = 1.D0
      CX = PS/RS
      U = SX*(G+A(NI+1))
      A(NI) = G+U
      NTOP = NX-2
C
C  THIS LOOP COMPLETES ONE ITERATION, THAT IS ONE QR TRANSFORM.
C
      DO 10 I=NI,NTOP
C
C  G IS THE GAMMA IN THE NOTATION OF WILKINSON.
C
        G = A(I+1)-U
        IF (CX.GT.TOL) GOTO 12
        PS = B(I)*CXS
        GOTO 16
12      PS = G**2/CX
16      RS = PS+B(I+1)
C
C  ROTATE AN OFF-DIAGONAL ELEMENT.
C
        B(I) = SX*RS
        SX = B(I+1)/RS
        CXS = CX
        CX = PS/RS
        U = SX*(G+A(I+2))
C
C  ROTATE A DIAGONAL ELEMENT.
C
        A(I+1) = G+U
10    CONTINUE
C
C  COMPUTE THE LAST DIAGONAL ELEMENT.
C
      A(NX) = A(NX)-U
C
C  COMPUTE THE LAST OFF-DIAGONAL ELEMENT.
C
      IF (CX.GT.TOL) GOTO 112
      PS = B(NTOP+1)*CXS
      GOTO 116
112   PS = ((A(NX))**2)/CX
116   B(NTOP+1) = SX*PS
C
C  END OF ONE ITERATION.
C
85    IT = NX
C
C  CHECK UPWARD THROUGH THE OFF-DIAGONAL ELEMENTS TO FIND THOSE LESS
C  THAN TOL. IF NO OFF-DIAGONAL ELEMENTS LESS THAN TOL ARE FOUND,
C  PERFORM ANOTHER ITERATION.
C
30    IT = IT-1
      IF (DABS(B(IT)).LE.TOL) GOTO 40
      IF (IT-NI) 100,100,30
C
C  BRANCH ACCORDING TO WHETHER THE MATRIX ISOLATED BY THE SMALL
C  OFF-DIAGONAL ELEMENT IS OF DIMENSION ONE, TWO, OR MORE.
C
40    IF (NX-IT-2) 50,60,70
C
C  EXTRACT THE EIGENVALUE OF A ONE BY ONE MATRIX BY ADDING BACK
C  THE SHIFT.
C
50    A(NX) = A(NX)+SH
C
C  DECREASE THE SIZE OF THE PORTION OF THE MATRIX AFFECTED BY
C  LATER ITERATIONS.
C
      NX = NX-1
C
C  RESET THE ITERATION COUNTER.
C
      K = 1
      GOTO 80
C
C  EXTRACT THE EIGENVALUES FROM A TWO BY TWO MATRIX.
C
60    AL = B(NX-1)
      AM = 5.D-1*(A(NX-1)-A(NX))
      AMS = AM**2
      SAM = DSIGN(1.D0,AM)
      AN = DSQRT(AL+AM**2)
      CX = (AN+DABS(AM))/(2.D0*AN)
      SX = B(NX-1)/(4.D0*AN**2*CX)
      TA = A(NX-1)
      TB = A(NX)
      TC = B(NX-1)
      I = NX
C
C  ROTATE THE DIAGONAL ELEMENTS AND THE OFF-DIAGONAL ELEMENTS.
C
      A(NX-1) = TA*CX+TB*SX+TC*SAM/AN+SH
      A(NX) = TA*SX+TB*CX-TC*SAM/AN+SH
      B(NX-1) = 4.D0*AMS*CX*SX-DABS(AM)*TC/AN+TC*(CX-SX)**2
C
C  RESET THE ITERATION COUNTER.
C
      K = 1
C
C  DECREASE THE SIZE OF THE PORTION OF THE MATRIX AFFECTED BY THE LATER
C  ITERATIONS.
C
      NX = NX-2
      GOTO 80
C
C  THE NEXT STATEMENT IS REACHED WHEN THE PORTION OF THE MATRIX
C  ISOLATED IS GREATER THAN TWO BY TWO.  IT CHANGES THE LOWER LIMIT
C  OF THE ITERATION SO THAT ONLY THIS PORTION WILL BE AFFECTED BY
C  SUBSEQUENT ROTATIONS UNTIL ALL ITS EIGENVALUES ARE FOUND.
C
70    NI = IT+1
C
C  TRANSFER TO BEGINNING OF ANOTHER ITERATION.
C
      GOTO 85
C
C  NEXT STATEMENT IS REACHED AFTER EITHER ONE OR TWO EIGENVALUES
C  HAVE JUST BEEN FOUND.  IT TRANSFERS IF ALL THE EIGENVALUES IN
C  THIS PORTION OF THE MATRIX HAVE BEEN FOUND.
C
80    IF (NX.LT.NI) GOTO 90
C
C  BRANCH ACCORDING TO WHETHER ONE, TWO, OR MORE EIGENVALUES REMAIN
C  TO BE FOUND.
C
95    IF (NX-NI-1) 50,60,85
C
C  THE NEXT STATEMENT IS REACHED WHEN ALL EIGENVALUES IN THIS PART
C  OF THE MATRIX HAVE BEEN FOUND. IT RETURNS IF THIS IS THE LAST
C  PART OF THE MATRIX.
C
90    IF (NI.EQ.1) RETURN
C
C  ENLARGE THE PORTION OF THE MATRIX BEING TREATED TO INCLUDE THE
C  BEGINNING OF THE MATRIX.
C
      NI = 1
      GOTO 95
      END
