C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE FMOT                                                 *
C *                                                                  *
C *  Routine of ICONC                                                *
C *                                                                  *
C *  Usage......: Actually fills fragment overlap and hamiltonian    *
C *               from the molecular matrices.                       *
C *                                                                  *
C *  Author ....: Charles Wilker (R. Hoffmann group)                 *
C *  Date.......: December 1981                                      *
C *  Revision...: last revision 24.08.97/mbs (Martin Braendle)       *
C *  Available..: Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *  Remarks....: Adapted from the BIGICON version.                  *
C *                                                                  *
C *  Called from: FMOS                                               *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE FMOT(H,S,H1,S1, NDIM,NATOM,ND1,NUMB)
 
      INCLUDE 'iconpara.for'
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      INTEGER ND1
      DIMENSION H(NDIM,NDIM),S(NDIM,NDIM)
      DIMENSION H1(ND1,ND1),S1(ND1,ND1)
 
      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
 
      COMMON/FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .           NFRAG(9),NUMBAT(9),IFMOPT(4*MXATOM),IBCKPT(MXATOM),
     .           PRIFMO(7)
      INTEGER FMO,FMOCH,SORA,SORB
      LOGICAL*1 PRIFMO
 
      IS=1
      NA=0
      DO 10 I=1,NATOM
        IF (FMO(I).NE.NUMB) GOTO 10
        NA=NA+1
        I1=SORB(I)-IS
        IE=SORB(I+1)-(I1+1)
 
        DO 13 J=IS,IE
          JJ=J+I1
          DO 13 K=J,IE
            KK=K+I1
            IF (J.EQ.K) GOTO 13
            S1(J,K)=S(JJ,KK)
            H1(K,J)=H(JJ,KK)
13          H1(J,K)=H(JJ,KK)
 
        LS=IE+1
        NB=NA
        II=I+1
        IF (II.GT.NATOM) GOTO 10
        DO 11 L=II,NATOM
          IF (FMO(L).NE.NUMB) GOTO 11
          NB=NB+1
          DELX=X(I)-X(L)
          DELY=Y(I)-Y(L)
          DELZ=Z(I)-Z(L)
          S1(NB,NA)=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
 
          L1=SORB(L)-LS
          LE=SORB(L+1)-(L1+1)
          DO 12 J=IS,IE
            JJ=J+I1
            DO 12 K=LS,LE
              KK=K+L1
              S1(J,K)=S(JJ,KK)
              H1(J,K)=H(JJ,KK)
              H1(K,J)=H(JJ,KK)
   12     CONTINUE
          LS=LE+1
   11   CONTINUE
        IS=IE+1
   10 CONTINUE
      NUMBAT(NUMB)=NA
 
      RETURN
      END
