C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE INPUT                                                *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: reads in ICON-inputfile, which has to be generated *
C *               by INPUTC.                                         *
C *                                                                  *
C *  Authors....: -                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: last revision 24.08.97/mbs    (Martin Braendle)    *
C *                             15.05.92/rr     (Ruedi Rytz)         *
C *  Remarks....: Reading in of FMO-specifications added.            *
C *               Writing of input information now in subroutine     *
C *               INPRT.                                             *
C *               Charge iteration and FMO calculation not allowed   *
C *               at the same time. 20.11.90/mbs                     *
C *               Reading in of k-parameter specification and omit-  *
C *               atom-from-repulsion-energy calculation added.      *
C *               25.06.91/mbs                                       *
C *               Inclusion of an orbital to atom number pointer     *
C *               IATMPT. 15.05.92/mbs                               *
C *               MOBY output implemented. 15.05.92/rr               *
C *               Reading in of user specified atomic electron con-  *
C *               figurations added.                                 *
C *                                                                  *
C *  Called from: ICON                                               *
C *                                                                  *
C *  Calls......: INPRT                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE INPUT(NATOM,NDIM,NTYPE,TEND)
      INCLUDE 'iconpara.for'

      IMPLICIT REAL*8(A-H,O-Z)

      PARAMETER(NDIMMX=4*MXATOM)

      COMMON /TITLE/AB(10)

      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY

      COMMON /CNTRL/CON,PEEP,COULH,NH,NA,NATM,KA,NELEC,METH,IPRINT,
     .              IPUNCH,L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
      LOGICAL*1     L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT

      COMMON /OUT/PRT(24),PUN(24),IOVPOP(24),IENRGY(24)
      LOGICAL*1 PRT,PUN
      INTEGER   IOVPOP,IENRGY

      COMMON/ANEWDI/CWK,CWKALP,R(MXATOM,MXATOM)

      COMMON/ITPARM/DAMP1,DAMP2,DAMP3,LAMPRI,DELTAC,SENSE,MAXCYC,
     .              PRTCYC,ICYCLE,NCON,PARTIT,PRINTX,ITABLE(MXITER)
      REAL*8 LAMPRI
      LOGICAL*1 PARTIT,PRINTX,ITABLE
      INTEGER*4 PRTCYC

      COMMON/FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .           NFRAG(9),NUMBAT(9),IFMOPT(4*MXATOM),IBCKPT(MXATOM),
     .           PRIFMO(7)
      INTEGER FMO,FMOCH,SORA,SORB
      LOGICAL*1 PRIFMO

      COMMON/KSET/KATOM(MXSETK,2),SETK(MXSETK),SETD(MXSETK),NKSET
      INTEGER NKSET,KATOM

      COMMON/OMITRP/NOMIT,OMIT(MXOMIT)
      INTEGER NOMIT,OMIT

      COMMON /ATOMPT/IATMPT(NDIMMX)

      COMMON /AOOCC/NAOOCC,ICONF(MXAOOC,4)

      CHARACTER*2 ACSNEW(MXATOM),CHANGE(MXATOM)
C
C obsolete, 7.12.95/mbs
C
C      REAL*4      MADS(MXITER),MADP(MXITER),MADD(MXITER)
      LOGICAL*1   TEND
C
C  Since the iternal atomic parameters ( EXPS, EXPP, etc. ) are
C  not used when doing charge iteration ( METH =1 ) the space
C  allocated to them can be used for the VSIE charge iteration
C  parameters.
C
C  obsolete and even wrong, 7.12.95/mbs
C
C      DIMENSION AS1(MXITER),BS1(MXITER),CS1(MXITER),AP1(MXITER),
C     .          BP1(MXITER),CP1(MXITER),AD1(MXITER),BD1(MXITER),
C     .          CD1(MXITER)
C      EQUIVALENCE (AS1(1),EXPS(MXITER+1)),(BS1(1),EXPP(MXITER+1)),
C     .            (CS1(1),EXPD(MXITER+1)),(AP1(1),EXPD2(MXITER+1)),
C     .            (BP1(1),C1(MXITER+1)),(CP1(1),C2(MXITER+1)),
C     .            (AD1(1),COULS(MXITER+1)),(BD1(1),COULP(MXITER+1)),
C     .            (CD1(1),COULD(MXITER+1))
C      EQUIVALENCE (MADS(1),NS(MXITER+1)),(MADP(1),NP(MXITER+1)),
C     .            (MADD(1),ND(MXITER+1))
C
C      COMMON /ABC/AS2(MXITER),BS2(MXITER),CS2(MXITER),AP2(MXITER),
C     .            BP2(MXITER),CP2(MXITER),AD2(MXITER),BD2(MXITER),
C     .            CD2(MXITER),AS3(MXITER),BS3(MXITER),CS3(MXITER),
C     .            AP3(MXITER),BP3(MXITER),CP3(MXITER),AD3(MXITER),
C     .            BD3(MXITER),CD3(MXITER)
C

      COMMON /ABC/AS1(MXITER),BS1(MXITER),CS1(MXITER),AP1(MXITER),
     .            BP1(MXITER),CP1(MXITER),AD1(MXITER),BD1(MXITER),
     .            CD1(MXITER),AS2(MXITER),BS2(MXITER),CS2(MXITER),
     .            AP2(MXITER),BP2(MXITER),CP2(MXITER),AD2(MXITER),
     .            BD2(MXITER),CD2(MXITER),AS3(MXITER),BS3(MXITER),
     .            CS3(MXITER),AP3(MXITER),BP3(MXITER),CP3(MXITER),
     .            AD3(MXITER),BD3(MXITER),CD3(MXITER),MADS(MXITER),
     .            MADP(MXITER),MADD(MXITER)
      REAL*8 MADS,MADP,MADD

      COMMON /STARS/STAR,STAR2,SYMBOL(MXATOM)
      CHARACTER*2 STAR,STAR2,SYMBOL

      COMMON /START/NUSER
C
C Read and write title.
C
      READ(5,10,END=580,ERR=580) AB
   10 FORMAT(8A8,A6,A2)
      WRITE(6,20) AB
   20 FORMAT('0',T10,8A8,A6,A2)
C
C Read parameter card.
C
C 23.10.90/mbs, 24.06.91/mbs, 17.10.91/mbs, 02.06.92/mbs
C
      READ(5,30,END=40) NH,NA,KA,METH,IPRINT,IPUNCH,L1,L2,L3,L4,L5,L6,
     .                  L7,L8,L9,CON,PEEP,COULH,(PRT(I),I=1,22)
   30 FORMAT(6I3,9L1,F5.2,2F6.3,22L1)
C
C Insert default parameters. New (8.6.1989): 1+kappa=2.00 and
C delta=0.35.
C
   40 READ(5,70) CWKALP
      IF (CWKALP .LT. 1.E-05) CWKALP=0.35D0
      IF (CON .LT. 1.E-05) CON=2.00D0
      CWK=CON-1
      IF (PEEP .LT. 1.E-05) PEEP=1.3D0
      IF (COULH .GT. -1.E-05) COULH=-13.6D0
      ITERAT=METH .NE. 0
      NATOM=NH+NA
      NATM=NATOM

C
C Set MOBY options.
C Output of coordinates, distance matrix, energy levels, wavefunctions
C and net charges will be used to create the MOBY file. Therefore
C these options have to be set. otherwise the needed values won' t
C be calculated!
C 15.05.92/rr
C
      IF ( PRT(22) ) THEN
        PRT(1) = .FALSE.
        PRT(3) = .FALSE.
        PRT(8) = .FALSE.
        PRT(10)= .FALSE.
        PRT(16)= .FALSE.
      ENDIF

C
C  SET IPRINT OPTION.
C
      IF (IPRINT .GT. 1) GOTO 50
      PRT(6)=.TRUE.
      PRT(7)=.TRUE.
      PRT(11)=.TRUE.
      PRT(17)=.TRUE.
      PRT(19)=.TRUE.
      PRT(21)=.TRUE.
      IF (IPRINT .GT. 0) GOTO 50
      PRT(12)=.TRUE.
      PRT(14)=.TRUE.
      PRT(20)=.TRUE.
      IF (IPRINT .GT. -1) GOTO 50
      PRT(13)=.TRUE.
      PRT(15)=.TRUE.
      PRT(16)=.TRUE.
      PRT(18)=.TRUE.
      IF (IPRINT .GT. -2) GOTO 50
      PRT(10)=.TRUE.
C
C Read coordinates and heavy atom card.
C
   50 IF (NATOM .GT. MXATOM) THEN
        WRITE(6,60) MXATOM
        STOP
      ENDIF
   60 FORMAT('1',T10,'MAX. NUMBER OF ATOMS (',I3,') EXCEEDED.'///)

      READ(5,70) (X(I),Y(I),Z(I),I=1,NATOM)
   70 FORMAT(3F15.6)
      READ(5,80) (ACSNEW(I),I=1,NA)
   80 FORMAT(40A2)
C
C Decode atom definition cards. If first atom is not a ' *', internal
C BLOCK DATA definitions in PROGRAM ICON will be used.
C
      NDIM=NH
      NTYPE=NH
      NELEC=NH-KA
      K=NUSER
      NUSER2=MXATOM
      IORB=1
      IF (METH .GE. 2) NUSER2=MXITER
      DO 180 I=1,NA
        IF (NUSER .GT. NUSER2) GOTO 120
        DO 90 J=NUSER,NUSER2
          JSAVE=J
          IF (ACSNEW(I) .EQ. SYMBOL(J)) GOTO 160
   90   CONTINUE
C
C Provision for user specified data.
C
        IF (ACSNEW(I) .EQ. STAR) GOTO 120
C
C provision for repetition of user data.
C
        IF (ACSNEW(I) .EQ. STAR2) GOTO 140
        WRITE(6,100) I,ACSNEW(I)
  100   FORMAT(//,T10,'HEAVY ATOM',I3,' NOT RECOGNIZED. SYMBOL ',A2)
        IF (METH .GE. 2) WRITE(6,110)
  110   FORMAT(/,T10,'REMEMBER IF USING METH > 1 ALL ATOMIC ',
     .         'PARAMETERS MUST BE DEFINED BY THE USER.')
        REWIND 7
        STOP
  120   NUSER=NUSER-1
        IF (NUSER .LT. 1) THEN
          WRITE(6,130) I-1
          STOP
  130     FORMAT('1',T10,'MORE THAN ',I3,' DIFFERENT USER DEFINED ',
     .           'ATOMS. USE "ATOM-SYMBOL" FOR IDENTICAL ATOMS.',///)
        ENDIF
  140   READ(5,150) SYMBOL(NUSER),VELEC(NUSER),NS(NUSER),EXPS(NUSER),
     .              COULS(NUSER),NP(NUSER),EXPP(NUSER),COULP(NUSER),
     .              ND(NUSER),EXPD(NUSER),COULD(NUSER),C1(NUSER),
     .              EXPD2(NUSER),C2(NUSER)
  150   FORMAT(A2,I3,3(I3,2F6.3),F6.4,F6.3,F6.4)
        JSAVE=NUSER
C
C Normalize user specified contracted d orbital.
C
        IF (C2(NUSER) .EQ. 0.) GOTO 160
        SNDUS=FLOAT(ND(NUSER))
        S=(4.D0*EXPD(NUSER)*EXPD2(NUSER)/(EXPD(NUSER)+EXPD2(NUSER))**2)
     .     **(DBLE(SNDUS)+.5D0)
        S=1.D0/DSQRT(C1(NUSER)**2+C2(NUSER)**2+(S+S)*C1(NUSER)*
     .    C2(NUSER))
        C1(NUSER)=S*C1(NUSER)
        C2(NUSER)=S*C2(NUSER)
  160   NELEC=NELEC+VELEC(JSAVE)
C
C AC as KEY are pointers to the parameter tables. 01.11.90/mbs
C
        AC(I)=JSAVE
        KEY(I)=JSAVE
        SORB(I)=IORB

        IOLDND=NDIM
        NDIM=NDIM+4
        IF (NP(JSAVE) .EQ. 0) NDIM=NDIM-3
        IF (ND(JSAVE) .NE. 0) NDIM=NDIM+5
        IORB=NDIM+1

        NTYPE=NTYPE+2
        IF (NP(JSAVE) .EQ. 0) NTYPE=NTYPE-1
        IF (ND(JSAVE) .NE. 0) NTYPE=NTYPE+1
        DO 170 J=IOLDND+1,NDIM
          IATMPT(J)=I
  170   CONTINUE
  180 CONTINUE
      SORB(I)=IORB
C
C Read in charge iteration parameters. Set default values.
C
      IF (.NOT. ITERAT) GOTO 360
      IF (K .NE. MXITER+1) GOTO 360
      READ(5,190) DAMP1,DAMP2,DAMP3,LAMPRI,DELTAC,SENSE,MAXCYC,PRTCYC,
     .            NCON,PARTIT
  190 FORMAT(6F10.5,3I5,4X,L1)
      IF ((.NOT. PARTIT) .OR. (METH .EQ. 2)) GOTO 210
      WRITE(6,200)
  200 FORMAT(///,T10,'PARTIAL ITERATION ( PARTIT=TRUE ) MAY ONLY BE ',
     .       'USED IF METH=2.')
      STOP
  210 IF (DELTAC .EQ. 0.0D0) DELTAC=0.0001D0
      IF (SENSE .EQ. 0.0D0) SENSE=2.0D0
      IF (MAXCYC .EQ. 0) MAXCYC=25
      IF (PRTCYC .EQ. 0) PRTCYC=MAXCYC
      IF (NCON .EQ. 0) NCON=3
      IF (DAMP1 .EQ. 0.0D0) DAMP1=0.1D0
      IF (METH .GE. 3) GOTO 220
      IF (DAMP2 .EQ. 0.0D0) DAMP2=0.25D0
      IF (LAMPRI .EQ. 0.0D0) LAMPRI=0.25D0
      GOTO 230
  220 IF (DAMP2 .EQ. 0.0D0) DAMP2=0.75D0
      IF (LAMPRI .EQ. 0.0D0) LAMPRI=0.75D0
  230 IF (METH .LT. 2) GOTO 360
      DO 240 I=1,20
  240 ITABLE(I)=.FALSE.
      NUSER2=MXITER+1-NUSER
C
C Read in symbols of atoms on which charge iteration is to be performed.
C
      IF ( .NOT. PARTIT) GOTO 270
      READ(5,250) (CHANGE(I),I=1,40)
  250 FORMAT(40A2)
      DO 260 I=1,NUSER2
        J=MXITER+1-I
        DO 260 K=1,NUSER2
          IF (SYMBOL(J) .EQ. CHANGE(K)) ITABLE(J)=.TRUE.
  260 CONTINUE
      GOTO 290
  270 DO 280 I=1,NUSER2
        J=MXITER+1-I
        ITABLE(J)=.TRUE.
  280 CONTINUE
C
C Read in VSIE and MADELUNG parameters.
C
  290 DO 350 I=1,NUSER2
        J=MXITER+1-I
        IF ( .NOT. ITABLE(J)) GOTO 350
        READ(5,300) AS1(I),BS1(I),CS1(I),MADS(I)
  300   FORMAT(4F10.8)
        IF (NP(J) .EQ. 0) GOTO 350
        IF (ND(J) .NE. 0) GOTO 310
        READ(5,300) AP1(I),BP1(I),CP1(I),MADP(I)
        GOTO 350
  310   IF (NCON .EQ. 3) GOTO 320
        READ(5,300) AP1(I),BP1(I),CP1(I),MADP(I),AD1(I),BD1(I),CD1(I),
     .              MADD(I)
        GOTO 350
  320   IF (I .GT. MXITER) THEN
          WRITE(6,330) MXITER,I
          STOP
        ENDIF
  330   FORMAT('1',T10,'ONLY ',I2,' ATOMS OF ATOM TYPE ',I3,' ALLOWED ',
     .         'AND THEY MUST BE THE FIRST ATOMS DEFINED IN THE INPUT.',
     .         //)
        READ(5,340) AS2(I),BS2(I),CS2(I),AS3(I),BS3(I),CS3(I)
  340   FORMAT(3F10.8)
        READ(5,300) AP1(I),BP1(I),CP1(I),MADP(I)
        READ(5,340) AP2(I),BP2(I),CP2(I),AP3(I),BP3(I),CP3(I)
        READ(5,300) AD1(I),BD1(I),CD1(I),MADD(I)
        READ(5,340) AD2(I),BD2(I),CD2(I),AD3(I),BD3(I),CD3(I)
  350 CONTINUE
C
C Read in FMO specifications and set NFRAG for FMO.
C 22.11.90/mbs, 22.8.97/mbs
C
  360 DO 365 I=1,NDIM
        IFMOPT(I)=I
  365 CONTINUE
      DO 367 I=1,NATOM
        IBCKPT(I)=I
  367 CONTINUE

      IF (L6) THEN
        READ(5,370) (FMO(I),I=1,NATOM)
  370   FORMAT(78I1)
        READ(5,380) (FMOCH(I),I=1,9)
  380   FORMAT(9I3)
        READ(5,390) (PRIFMO(I),I=1,7)
  390   FORMAT(7L1)
        DO 400 I=1,9
          NFRAG(I)=0
  400   CONTINUE
        DO 410 I=1,NATOM
          J=FMO(I)
          NFRAG(J)=NFRAG(J)+SORB(I+1)-SORB(I)
  410   CONTINUE
      ENDIF
C
C Read in k-parameter specifications.
C 24.06.91/mbs
C
      IF (L7) THEN
        READ(5,420) NKSET
  420   FORMAT(BN,I3)
        DO 440 I=1,NKSET
          READ(5,430) KATOM(I,1),KATOM(I,2),SETK(I),SETD(I)
  430   FORMAT(BN,2I4,2F10.5)
  440   CONTINUE
      ENDIF
C
C Read in omit-atom specifications.
C 24.06.91/mbs
C
      IF (L8) THEN
        READ(5,450) NOMIT,(OMIT(I),I=1,NOMIT)
  450   FORMAT(BN,11I4)
      ENDIF
C
C Read in user specified atomic electron configurations.
C 02.06.92/mbs
C
      IF (L9) THEN
        READ(5,452) NAOOCC
  452   FORMAT(I2)
        IF (NAOOCC .GT. 0) THEN
          DO 457 I=1,NAOOCC
            READ(5,455) (ICONF(I,J),J=1,4)
  455       FORMAT(BN,4I3)
  457     CONTINUE
        ENDIF
      ENDIF
C
C Read in IOVPOP(I) and IENRGY(I). Individulal overlap population
C analyses are performed from orbital IOVPOP(N) to orbital IOVPOP(N+1).
C Individual energy matrix analyses are performed from orbital
C IENRGY(N) to orbital IENRGY(N+1).
C
      IF (L3) READ(5,460) IOVPOP
  460 FORMAT(24I3)
      IF (L4) READ(5,460) IENRGY
C
C Write atom definitions. 24.10.90/mbs
C
      IF (PRT(1)) GOTO 470
      NCHG=KA
      CALL INPRT(NATOM,NCHG,0)
C
C Print out iteration parameters.
C
  470 IF (.NOT. ITERAT) GOTO 560
      WRITE(6,480) DAMP1,DAMP2,DAMP3,LAMPRI,MAXCYC,PRTCYC,SENSE,DELTAC
  480 FORMAT(/,T10,'DAMP1 =',F6.3,6X,'DAMP2 =',F6.3,6X,'DAMP3 =',F6.3,
     .       6X,'LAMPRI =',F6.3,//,T10,'MAXCYC =',I3,8X,'PRTCYC =',I3,
     .       8X,'SENSE =',F6.3,6X,'DELTAC =',F10.7)
C
C Print out vsie parameters.
C
      IF (METH .LT. 2) GOTO 560
      WRITE(6,490)
  490 FORMAT(///,' VSIE PARAMETERS',//,T10,'ATOM',T26,'A',T39,'B',T52,
     .       'C')
      NUSER2=MXITER+1-NUSER
      DO 540 I=1,NUSER2
        J=MXITER+1-I
        IF ( .NOT. ITABLE(J)) GOTO 540
        WRITE(6,500) SYMBOL(J),AS1(I),BS1(I),CS1(I)
  500   FORMAT(/,T11,A2,4X,3F13.5)
        IF (NP(J) .EQ. 0) GOTO 540
        IF (ND(J) .NE. 0) GOTO 520
        WRITE(6,510) AP1(I),BP1(I),CP1(I)
  510   FORMAT(T17,3F13.5)
        GOTO 540
  520   IF (NCON .EQ. 3) GOTO 530
        WRITE(6,510) AP1(I),BP1(I),CP1(I),AD1(I),BD1(I),CD1(I)
        GOTO 540
  530   WRITE(6,510) AS2(I),BS2(I),CS2(I),AS3(I),BS3(I),CS3(I),AP1(I),
     .               BP1(I),CP1(I),AP2(I),BP2(I),CP2(I),AP3(I),BP3(I),
     .               CP3(I),AD1(I),BD1(I),CD1(I),AD2(I),BD2(I),CD2(I),
     .               AD3(I),BD3(I),CD3(I)
  540 CONTINUE

      IF (L6) THEN
        ITERAT=.FALSE.
        METH=0
        WRITE(6,550)
  550   FORMAT(//,' CHARGE ITERATION WIILL NOT BE DONE BECAUSE OF FMO ',
     .            'CALCULATION.')
      ENDIF

  560 WRITE(6,570)
  570 FORMAT(///)
      RETURN

  580 TEND=.TRUE.
      RETURN
      END
