C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE MOVLAP                                               *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: calculates interatomic distances, overlap matrix,  *
C *               and madelung parameters.                           *
C *                                                                  *
C *  Authors....: -                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: last revision 17.10.91/mbs (Martin Braendle)       *
C *  Remarks....: COMMON block FMO added for FMO specifications and  *
C *               SORB. Setting of SORB still done in subroutine     *
C *               INPUT. Writing of distance and overlap matrix will *
C *               now be done in new subroutine OVPRT.               *
C *                                                                  *
C *  Called from: MATRIX                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               DELETS                                             *
C *               VALMAD                                             *
C *               OVPRT                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE MOVLAP(S,MAD,C,SP,PD,MAXS,MAXP,MAXD,COUL0,NDIM,NC,
     .                  NATOM,NTYPE)

      INCLUDE 'iconpara.for'

      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION S(NDIM,NDIM),MAD(NTYPE,NTYPE),C(NC),SP(NDIM),PD(NDIM),
     .          MAXS(NDIM),MAXP(NDIM),MAXD(NDIM),COUL0(20)
      INTEGER*4 COUL0
      REAL*8    MAD
      LOGICAL*4 SP,PD
C
C COUL0 dimensioned at 20 for easy reading during processing of
C deletion input. deletions done in subroutine DELETS.
C
      COMMON /TITLE/ AB(10)

      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
C
C Names L1,L2 of COMMON block CNTRL changed to LA,LB because of COMMON
C block LOCLAP.
C
      COMMON /CNTRL/CON,PEEP,COULH,NH,NA,NATM,KA,NELEC,METH,IPRINT,
     .              IPUNCH,LA,LB,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
      LOGICAL*1 LA,LB,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT

      COMMON /FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .            NFRAG(9),NUMBAT(9),PRIFMO(7)
      INTEGER   FMO,FMOCH,SORA,SORB,NFRAG,NUMBAT
      LOGICAL*1 PRIFMO

      COMMON /OUT/PRT(24),PUN(24),IOVPOP(24),IENRGY(24)
      INTEGER   IOVPOP,IENRGY
      LOGICAL*1 PRT,PUN

      COMMON /STARS/STAR,STAR2,SYMBOL(MXATOM)
      CHARACTER*2 STAR,STAR2,SYMBOL

      COMMON /LOCLAP/SK1,SK2,R,L1,L2,M,N1,N2,MAX

      REAL*4 MADS(MXITER),MADP(MXITER),MADD(MXITER)
      EQUIVALENCE (MADS(1),NS(MXITER+1)),(MADP(1),NP(MXITER+1)),
     .            (MADD(1),ND(MXITER+1))
                                                                  
      LOGICAL*1 JGO,FLAG

      DIMENSION PTR(9),DTR(25)
      EQUIVALENCE (PTR(3),CA),(PTR(8),CB)
      DIMENSION A(20),B(20),A1(20),B1(20)

      DATA SQRT3 /1.7320508075688770/
      DATA AUI /1.889644746D0/
      DATA PTR(9) /0.D0/,DTR(12)/0.D0/,DTR(22)/0.D0/

      NH1=NH+1
C
C Hydrogen-hydrogen overlaps.
C
      IF (NH .LE. 1) GOTO 30
      DO 20 I=2,NH
        IM1=I-1
        DO 20 J=1,IM1
          R=DSQRT((X(I)-X(J))**2+(Y(I)-Y(J))**2+(Z(I)-Z(J))**2)
C
C Store overlaps in upper right triangle of S(I,J). Put distances in
C lower right triangle.
C
          S(I,J)=R
          R=R*PEEP*AUI
          IF (R .GT. 50) GOTO 10
          SIGMA=(1.D0+R*(1.D0+R/3.D0))/DEXP(R)
          GOTO 20
   10     SIGMA=0.D0
   20 S(J,I)=SIGMA
C
C Heavy atom-heavy atom overlaps. Local coordinate system centered on
C atom J. Fill in upper right triangle of S(I,J).
C
   30 DO 570 I=1,NA
        INH=I+NH
        IM1=I-1
        KEYI=AC(I)
        MAXD(I)=ND(KEYI)
        MAXP(I)=NP(KEYI)
        MAXS(I)=NS(KEYI)
        SP(I)=EXPS(KEYI) .EQ. EXPP(KEYI)
        PD(I)=EXPP(KEYI) .EQ. EXPD(KEYI)
        IF (PD(I)) MAXP(I)=MAX0(MAXP(I),MAXD(I))
        IF (SP(I)) MAXS(I)=MAX0(NS(KEYI),MAXP(I))
C
C  SORB(I) contains a pointer to the s orbital on atom I.
C
        IORBS=SORB(I)
        IORB=SORB(I+1)-1
        IF (NP(KEYI) .EQ. 0) GOTO 50
        JD1=IORB-1
        DO 40 JC=IORBS,JD1
          ID=JC+1
          DO 40 IC=ID,IORB
   40   S(JC,IC)=0.D0
   50   CONTINUE
        IF (I .EQ. 1) GOTO 500
        DO 490 J=1,IM1
          KEYJ=AC(J)
          JNH=J+NH
          JORBS=SORB(J)
          DELX=X(INH)-X(JNH)
          DELY=Y(INH)-Y(JNH)
          DELZ=Z(INH)-Z(JNH)
          RT2=DELX**2+DELY**2
          R=DSQRT(RT2+DELZ**2)
          S(INH,JNH)=R
C
C Store distances in lower left triangle of S(I,J).
C
          IF (R .GT. 0.0D0) GOTO 70
          ID=IORB-1
          JD=SORB(J+1)-1
          DO 60 IC=IORBS,ID
            DO 60 JC=JORBS,JD
   60     S(JC,IC)=0.0D0
          GOTO 490
   70     IF (RT2 .GT. 1.E-10) GOTO 80
          CB=1.D0
          SB=0.D0
          SA=0.D0
          GOTO 90
   80     T=DSQRT(RT2)
          CB=DELX/T
          SB=DELY/T
          SA=T/R
   90     CA=DELZ/R
C
C The transformation matrices are calculated explicitly.
C PTR is the matrix for projecting the x,y,z orbitals onto the local
C system. the elements are ordered so that first x then y then z is 
C projected onto the z' axis (sigma). Then the 3 are projected onto 
C the x' axis and then the y' (pi). The d orbitals are handled 
C similarly. the order of projection is x2-y2,z2,xy,xz,yz first onto 
C z2'(sigma)and then onto xz' and yz'(pi). Finally the 5 orbitals are 
C projected onto x'2-y'2 and then xy' (delta).
C
C Those PTR and DTR which are zero are initialized in a DATA statement.
C CA and CB have been equivalenced to PTR(3) and PTR(8) respectively to
C save time.
C
          PTR(1)=SA*CB
          PTR(2)=SA*SB
C ...     PTR(3)= CA
          PTR(4)=CA*CB
          PTR(5)=CA*SB
          PTR(6)=-SA
          PTR(7)=-SB
C ...     PTR(8)= CB
C ...     PTR(9)= 0.D0
          IF (ND(KEYI)+ND(KEYJ) .EQ. 0) GOTO 100
          CA2=CA**2
          SA2=SA*SA
          CB2=CB*CB
          SB2=SB*SB
          CBSB=CB*SB
          CASA=CA*SA
          CB2SB2=CB2-SB2
          DTR(1)=SQRT3*.5D0*SA2*CB2SB2
          DTR(2)=1.D0-1.5D0*SA2
          DTR(3)=SQRT3*CBSB*SA2
          DTR(4)=SQRT3*CASA*CB
          DTR(5)=SQRT3*CASA*SB
          DTR(6)=CASA*CB2SB2
          DTR(7)=-SQRT3*CASA
          DTR(8)=2.D0*CASA*CBSB
          DTR(9)=CB*(CA2-SA2)
          DTR(10)=SB*(CA2-SA2)
          DTR(11)=-2.D0*SA*CBSB
C ...     DTR(12)= 0.D0
          DTR(13)=SA*CB2SB2
          DTR(14)=-PTR(5)
          DTR(15)=PTR(4)
          IF (ND(KEYI)*ND(KEYJ) .EQ. 0) GOTO 100
          DTR(16)=.5D0*(1.D0+CA2)*CB2SB2
          DTR(17)=.5D0*SQRT3*SA2
          DTR(18)=CBSB*(1.D0+CA2)
          DTR(19)=-CASA*CB
          DTR(20)=-CASA*SB
          DTR(21)=-2.D0*CA*CBSB
C ...     DTR(22)= 0.D0
          DTR(23)=CA*CB2SB2
          DTR(24)=PTR(2)
          DTR(25)=-PTR(1)
  100     R=R*AUI
C
C (s(I)^s(J)).
C
          N2=NS(KEYJ)
          N1=NS(KEYI)
          L2=0
          L1=0
          M=0
          MAX=MAXS(I)+MAXS(J)
          SK1=EXPS(KEYI)
          SK2=EXPS(KEYJ)
          CALL ABFNS(A,B)
          CALL LOVLAP(SIGMA,A,B)
          S(JORBS,IORBS)=SIGMA
C
C if the s exponent of atom I equals the p exponent we need not 
C calculate the A and B functions again.
C
C (p(I)^s(J)).
C
          JGO=.FALSE.
          IF (KEYI .EQ. KEYJ) GOTO 200
          IF (( .NOT. SP(I)) .OR. (NP(KEYI) .EQ. 0)) GOTO 200
  110     N1=NP(KEYI)
          L1=1
          CALL LOVLAP(SIGMA,A,B)
          SIGMA=-SIGMA
          DO 120 IC=1,3
  120     S(JORBS,IORBS+IC)=PTR(IC)*SIGMA
          IF (PD(I)) GOTO 130
          IF (JGO) GOTO 430
          GOTO 190
C
C (d(I)^s(J)) conditionally at first chance.
C
  130     N1=ND(KEYI)
          L1=2
          CALL LOVLAP(SIGMA,A,B)
          IF (C2(KEYI) .EQ. 0.D0) GOTO 140
          SK1=EXPD2(KEYI)
          CALL ABFNS(A1,B1)
          CALL LOVLAP(PART2,A1,B1)
          SIGMA=C1(KEYI)*SIGMA+C2(KEYI)*PART2
          SK1=EXPD(KEYI)
  140     ID=IORBS+3
          DO 150 IC=1,5
  150     S(JORBS,ID+IC)=DTR(IC)*SIGMA
C
C Calculate (d(I)^p(J)) if one can use same A's and B's.
C
          IF (SP(J)) GOTO 160
          IF (JGO) GOTO 450
          GOTO 190
  160     N2=NP(KEYJ)
          L2=1
          M=0
          CALL LOVLAP(SIGMA,A,B)
          M=1
          CALL LOVLAP(PI,A,B)
          IF (C2(KEYI) .EQ. 0.D0) GOTO 170
          SK1=EXPD2(KEYI)
          CALL LOVLAP(PART2,A1,B1)
          PI=C1(KEYI)*PI+C2(KEYI)*PART2
          M=0
          CALL LOVLAP(PART2,A1,B1)
          SK1=EXPD(KEYI)
          SIGMA=C1(KEYI)*SIGMA+C2(KEYI)*PART2
  170     PI=-PI
          ID=IORBS+3
          DO 180 JC=1,3
            DO 180 IC=1,5
  180     S(JORBS+JC,ID+IC)=PTR(JC)*DTR(IC)*SIGMA+(PTR(JC+3)*
     .                         DTR(IC+5)+PTR(JC+6)*DTR(IC+10))*PI
          IF (JGO) GOTO 490
C
C Now test for duplicate exponents on atom J. However do calculations
C anyhow.
C
  190     N1=NS(KEYI)
          L1=0
C
C (s(I)^p(J)).
C
  200     IF (SP(J)) GOTO 210
          IF (NP(KEYJ) .EQ. 0) GOTO 410
          MAX=MAXS(I)+MAXP(J)
          SK2=EXPP(KEYJ)
          CALL ABFNS(A,B)
  210     N2=NP(KEYJ)
          L2=1
          M=0
          CALL LOVLAP(SIGMA,A,B)
          DO 220 IC=1,3
  220     S(JORBS+IC,IORBS)=PTR(IC)*SIGMA
          IF (SP(I)) GOTO 250
          JGO=.TRUE.
          IF (ND(KEYJ) .NE. 0) GOTO 270
C
C Branch to test for EXPP(J) .EQ. EXPD(J). Calculate (s^d) anyhow.
C Return will be made to the next statement.
C
C (p(I)^p(J))   EXPP(I) EQ,NE EXPS(I).
C
          GOTO 240
  230     N2=NP(KEYJ)
          L2=1
          SK2=EXPP(KEYJ)
  240     IF (NP(KEYI) .EQ. 0) GOTO 410
          SK1=EXPP(KEYI)
C
C These statements are used only if one has already calculated
C (s(I)^d(J)) which means that SP(I) is false.
C
          MAX=MAXP(I)+MAXP(J)
          CALL ABFNS(A,B)
  250     N1=NP(KEYI)
          L1=1
          M=0
          CALL LOVLAP(SIGMA,A,B)
          SIGMA=-SIGMA
          M=1
          CALL LOVLAP(PI,A,B)
          DO 260 JC=1,3
            DO 260 IC=JC,3
              S(JORBS+JC,IORBS+IC)=PTR(JC)*PTR(IC)*SIGMA+(PTR(JC+3)
     .                                *PTR(IC+3)+PTR(JC+6)*PTR(IC+6))*
     .                                PI
  260     S(JORBS+IC,IORBS+JC)=S(JORBS+JC,IORBS+IC)
          IF (ND(KEYJ) .EQ. 0) GOTO 410
C
C  Branch around (s(I)^d(J)) if already done.
C
          IF (JGO) GOTO 310
C
C (s(I)^d(J)).
C
          N1=NS(KEYI)
          L1=0
  270     N2=ND(KEYJ)
          L2=2
          IF (PD(J)) GOTO 280
          SK2=EXPD(KEYJ)
          MAX=MAXS(I)+MAXD(J)
          CALL ABFNS(A,B)
  280     M=0
          CALL LOVLAP(SIGMA,A,B)
          IF (C2(KEYJ) .EQ. 0.D0) GOTO 290
          SK2=EXPD2(KEYJ)
          CALL ABFNS(A1,B1)
          CALL LOVLAP(PART2,A1,B1)
          SIGMA=C1(KEYJ)*SIGMA+C2(KEYJ)*PART2
          SK2=EXPD(KEYJ)
  290     JD=JORBS+3
          DO 300 IC=1,5
  300     S(JD+IC,IORBS)=DTR(IC)*SIGMA
          IF (JGO) GOTO 230
C
C SP(I) is true if here so branch as we also have d on atom J.
C
          GOTO 330
  310     JGO=.FALSE.
C
C (p(I)^d(J)).
C
          N2=ND(KEYJ)
          L2=2
          IF (PD(J)) GOTO 320
          SK2=EXPD(KEYJ)
          MAX=MAXP(I)+MAXD(J)
          CALL ABFNS(A,B)
  320     IF (C2(KEYJ) .EQ. 0.D0) GOTO 330
          SK2=EXPD2(KEYJ)
          CALL ABFNS(A1,B1)
          SK2=EXPD(KEYJ)
  330     N1=NP(KEYI)
          L1=1
          M=0
          CALL LOVLAP(SIGMA,A,B)
          M=1
          CALL LOVLAP(PI,A,B)
          IF (C2(KEYJ) .EQ. 0.D0) GOTO 340
          SK2=EXPD2(KEYJ)
          CALL LOVLAP(PART2,A1,B1)
          PI=C1(KEYJ)*PI+C2(KEYJ)*PART2
          M=0
          CALL LOVLAP(PART2,A1,B1)
          SIGMA=C1(KEYJ)*SIGMA+C2(KEYJ)*PART2
          SK2=EXPD(KEYJ)
  340     SIGMA=-SIGMA
          DO 350 IC=1,3
            DO 350 JC=1,5
  350     S(JD+JC,IORBS+IC)=DTR(JC)*PTR(IC)*SIGMA+(DTR(JC+5)*PTR(IC+3)+
     .                      DTR(JC+10)*PTR(IC+6))*PI
C
C (d(I)^d(J)).
C
          IF (ND(KEYI) .EQ. 0) GOTO 410
          MAX=MAXD(I)+MAXD(J)
          IF (PD(I)) GOTO 360
          SK1=EXPD(KEYI)
          CALL ABFNS(A,B)
          IF (C2(KEYJ) .EQ. 0.D0) GOTO 360
          SK2=EXPD2(KEYJ)
          CALL ABFNS(A1,B1)
          SK2=EXPD(KEYJ)
  360     N1=ND(KEYI)
          L1=2
          M=0
          CALL LOVLAP(SIGMA,A,B)
          M=1
          CALL LOVLAP(PI,A,B)
          M=2
          CALL LOVLAP(DELTA,A,B)
          CC=C2(KEYI)
          IF (C2(KEYJ) .EQ. 0.D0) GOTO 370
          CC=C1(KEYJ)*CC
          SK2=EXPD2(KEYJ)
          CALL LOVLAP(PART2,A1,B1)
          DELTA=C1(KEYJ)*DELTA+C2(KEYJ)*PART2
          M=1
          CALL LOVLAP(PART3,A1,B1)
          PI=C1(KEYJ)*PI+C2(KEYJ)*PART3
          M=0
          CALL LOVLAP(PART4,A1,B1)
          SIGMA=C1(KEYJ)*SIGMA+C2(KEYJ)*PART4
          SK2=EXPD(KEYJ)
          M=2
  370     IF (C2(KEYI) .EQ. 0.D0) GOTO 390
          IF (KEYI .EQ. KEYJ) GOTO 380
          SK1=EXPD2(KEYI)
          CALL ABFNS(A1,B1)
          CALL LOVLAP(PART2,A1,B1)
          M=1
          CALL LOVLAP(PART3,A1,B1)
          CALL LOVLAP(PART4,A1,B1)
  380     SIGMA=C1(KEYI)*SIGMA+CC*PART4
          PI=C1(KEYI)*PI+CC*PART3
          DELTA=C1(KEYI)*DELTA+CC*PART2
          IF (C2(KEYJ) .EQ. 0.D0) GOTO 390
          SK1=EXPD2(KEYI)
          SK2=EXPD2(KEYJ)
          CALL ABFNS(A1,B1)
          M=0
          CALL LOVLAP(PART2,A1,B1)
          CC=C2(KEYI)*C2(KEYJ)
          SIGMA=SIGMA+CC*PART2
          M=1
          CALL LOVLAP(PART2,A1,B1)
          PI=PI+CC*PART2
          M=2
          CALL LOVLAP(PART2,A1,B1)
          DELTA=DELTA+CC*PART2
  390     PI=-PI
          JD=JORBS+3
          DO 400 IC=1,5
            ID=IORBS+3
            DO 400 JC=1,5
              S(JD+JC,ID+IC)=DTR(IC)*DTR(JC)*SIGMA+(DTR(IC+5)*DTR(JC+5)+
     .                       DTR(IC+10)*DTR(JC+10))*PI+
     .                       (DTR(IC+15)*DTR(JC+15)+
     .                        DTR(IC+20)*DTR(JC+20))*DELTA
  400     S(JD+IC,ID+JC)=S(JD+JC,ID+IC)
C
C Filling in other half of overlaps for (J^I) as needed.
C
  410     IF (KEYI .EQ. KEYJ) GOTO 460
          N2=NS(KEYJ)
          L2=0
          SK2=EXPS(KEYJ)
          M=0
          JGO=.TRUE.
          IF (NP(KEYI) .EQ. 0) GOTO 490
          IF (SP(I)) GOTO 420
          MAX=MAXP(I)+MAXS(J)
          SK1=EXPP(KEYI)
          CALL ABFNS(A,B)
          GOTO 110
  420     IF (PD(I)) GOTO 440
  430     IF (ND(KEYI) .EQ. 0) GOTO 490
          MAX=MAXD(I)+MAXS(J)
          SK1=EXPD(KEYI)
          CALL ABFNS(A,B)
          GOTO 130
  440     IF (SP(J)) GOTO 490
          N1=ND(KEYI)
          L1=2
          SK1=EXPD(KEYI)
  450     IF (NP(KEYJ) .EQ. 0) GOTO 490
          SK2=EXPP(KEYJ)
          MAX=MAXD(I)+MAXP(J)
          CALL ABFNS(A,B)
          IF (C2(KEYI) .EQ. 0.D0) GOTO 160
          SK1=EXPD2(KEYI)
          CALL ABFNS(A1,B1)
          SK1=EXPD(KEYI)
          GOTO 160
  460     IF (NP(KEYI) .EQ. 0) GOTO 490
          DO 470 IC=1,3
  470     S(JORBS,IORBS+IC)=-S(JORBS+IC,IORBS)
          IF (ND(KEYI) .EQ. 0) GOTO 490
          DO 480 IC=4,8
            S(JORBS,IORBS+IC)=S(JORBS+IC,IORBS)
            DO 480 JC=1,3
  480     S(JORBS+JC,IORBS+IC)=-S(JORBS+IC,IORBS+JC)
  490   CONTINUE
C
C Calculate overlap integrals of hydrogen atoms.
C
  500   IF (NH .EQ. 0) GOTO 570
        N2=1
        L2=0
        M=0
        SK2=PEEP
        DO 560 J=1,NH
          DELX=X(J)-X(INH)
          DELY=Y(J)-Y(INH)
          DELZ=Z(J)-Z(INH)
          RT2=DELX**2+DELY**2
          R=DSQRT(RT2+DELZ**2)
C
C Store distances in lower left triangle of S(I,J).
C
          S(INH,J)=R
          IF (RT2 .GT. 1.D-10) GOTO 510
          CB=1.D0
          SB=0.D0
          SA=0.D0
          GOTO 520
  510     T=DSQRT(RT2)
          CB=DELX/T
          SB=DELY/T
          SA=T/R
  520     CA=DELZ/R
          R=R*AUI
C
C h(J)^s(I)).
C
          N1=NS(KEYI)
          L1=0
          MAX=1+MAXS(I)
          SK1=EXPS(KEYI)
          CALL ABFNS(A,B)
          CALL LOVLAP(SIGMA,A,B)
          S(J,IORBS)=SIGMA
          IF (NP(KEYI) .EQ. 0) GOTO 560
          IF (SP(I)) GOTO 530
          SK1=EXPP(KEYI)
          MAX=1+MAXP(I)
          CALL ABFNS(A,B)
  530     N1=NP(KEYI)
          L1=1
          CALL LOVLAP(SIGMA,A,B)
          S(J,IORBS+3)=CA*SIGMA
          SIGMA=SIGMA*SA
          S(J,IORBS+2)=SB*SIGMA
          S(J,IORBS+1)=CB*SIGMA
          IF (ND(KEYI) .EQ. 0) GOTO 560
          IF (PD(I)) GOTO 540
          SK1=EXPD(KEYI)
          MAX=1+ND(KEYI)
          CALL ABFNS(A,B)
  540     N1=ND(KEYI)
          L1=2
          CALL LOVLAP(SIGMA,A,B)
          IF (C2(KEYI) .EQ. 0.D0) GOTO 550
          SK1=EXPD2(KEYI)
          CALL ABFNS(A1,B1)
          CALL LOVLAP(PART2,A1,B1)
          SK1=EXPD(KEYI)
          SIGMA=C1(KEYI)*SIGMA+C2(KEYI)*PART2
  550     CONTINUE
          S(J,IORBS+5)=(1.D0-1.5D0*SA*SA)*SIGMA
          SIGMA=SIGMA*SQRT3*SA
          S(J,IORBS+4)=.5D0*SA*(CB*CB-SB*SB)*SIGMA
          S(J,IORBS+6)=CB*SB*SA*SIGMA
          SIGMA=SIGMA*CA
          S(J,IORBS+7)=CB*SIGMA
          S(J,IORBS+8)=SB*SIGMA
  560   CONTINUE
  570 CONTINUE
C
C Call DELETS to set certain overlap integrals=0.
C
      IF (LB) THEN
        CALL DELETS (S,COUL0,NDIM)
        WRITE (6,580)
  580 FORMAT(///)
      END IF
C
C Calculate interatomic MADELUNG parameters.
C
      IF (METH .LT. 3) GOTO 650
      IC=1
      DO 640 I=1,NA
        KEYI=AC(I)
        RS=0.0D0
        ID=IC
        MAD(ID,ID)=DBLE(MADS(MXITER+1-KEYI))
        IF (NP(KEYI) .EQ. 0) GOTO 610
        ID=IC+1
        MAD(ID,ID)=DBLE(MADP(MXITER+1-KEYI))
        IF (ND(KEYI) .EQ. 0) GOTO 590
        ID=IC+2
        MAD(ID,ID)=DBLE(MADD(MXITER+1-KEYI))
  590   M=IC+1
        DO 600 K=M,ID
          K1=K-1
          DO 600 L=IC,K1
            CA=MAD(K,K)
            CB=MAD(L,L)
            SA=VALMAD(CA,CB,RS)
            MAD(K,L)=SA
  600   MAD(L,K)=SA
  610   IF (I .EQ. 1) GOTO 640
        IM1=I-1
        JC=1
        DO 630 J=1,IM1
          KEYJ=AC(J)
          RS=S(I,J)*AUI/27.21D0
          JD=JC
          IF (NP(KEYJ) .NE. 0) JD=JC+1
          IF (ND(KEYJ) .NE. 0) JD=JC+2
          DO 620 K=IC,ID
            CA=MAD(K,K)
            DO 620 L=JC,JD
              CB=MAD(L,L)
              SA=VALMAD(CA,CB,RS)
              MAD(K,L)=SA
  620     MAD(L,K)=SA
  630   JC=JD+1
  640 IC=ID+1
C
C Print overlap and distance matrices.
C
  650 FLAG=.TRUE.
      CALL OVPRT(S,MAD,C,NDIM,NC,NATOM,NTYPE,FLAG)
      RETURN
      END
