C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  ORBRAD                                                          *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Calculates the orbital radii of the supplied ele-  *
C *               ments.                                             *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs), c/o                         *
C *               Calzaferri group,                                  *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne,                               *
C *               Freiestr. 3,                                       *
C *               CH-3012 Berne                                      *
C *  Date.......: 15.05.92                                           *
C *  Revision...: 05.05.93/mbs                                       *
C *  Remarks....: S. G.Calzaferri et al, J. Phys. Chem. 93(1989)5366 *
C *                                                                  *
C *  Called from: HUCKEL                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE ORBRAD(NATOM)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'iconpara.for'
 
      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
 
      COMMON /ORBRD/D0(4*MXATOM)
 
      A0=0.52917706D0
      IORB=1
      DO 30 I=1,NATOM
        KEYI=KEY(I)
        IF (NS(KEYI) .GT. 0) THEN
          D0(IORB)=DBLE(NS(KEYI))/EXPS(KEYI)*A0
          IORB=IORB+1
        ENDIF
 
        IF (NP(KEYI) .GT. 0) THEN
          DO 10 J=IORB,IORB+2
            D0(J)=DBLE(NP(KEYI))/EXPP(KEYI)*A0
   10     CONTINUE
          IORB=IORB+3
        ENDIF
 
        IF (ND(KEYI) .GT. 0) THEN
          DO 20 J=IORB,IORB+4
            IF (EXPD2(KEYI) .EQ. 0.0D0) THEN
              D0(J)=DBLE(ND(KEYI))/EXPD(KEYI)*A0
            ELSE
              D01=C1(KEYI)*C1(KEYI)*EXPD(KEYI)
              D02=C2(KEYI)*C2(KEYI)*EXPD2(KEYI)
              N2=2*ND(KEYI)
              D03=DBLE(2**N2)*(EXPD(KEYI)*EXPD2(KEYI))**
     .            (DBLE(ND(KEYI)+0.5D0))
              D04=(EXPD(KEYI)+EXPD2(KEYI))**DBLE(N2)
              D05=D03/D04
              D0(J)=DBLE(ND(KEYI))*A0/(D01+D02+D05)
            ENDIF
   20     CONTINUE
          IORB=IORB+5
        ENDIF
 
   30 CONTINUE
      RETURN
      END
