C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE OUTPUT                                               *
C *                                                                  *
C *  Routine of ICONC                                                *
C *                                                                  *
C *  Usage......: Calculates and prints out overlap population,      *
C *               charge, and energy partitioning matrices.          *
C *                                                                  *
C *  Authors....: -                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: last revisions 24.08.97/mbs (Martin Braendle)      *
C *                              15.05.92/rr  (Ruedi Rytz)           *
C *  Remarks....: Algoritms, variables and COMMON blocks for prin-   *
C *               ting out FMO results added.                        *
C *               Output of FMO occupation for given MO occupation   *
C *               changed. 27.11.90/mbs                              *
C *               COMMON block OUT changed. Outputting of core-core  *
C *               repulsion energies between atom pairs added.       *
C *               17.10.91/mbs                                       *
C *               MOBY output implemented/rr                         *
C *               Energy matrices corrected./mbs                     *
C *                                                                  *
C *  Called from: HUCK,HUCKEL                                        *
C *                                                                  *
C *  Calls......: DESORB                                             *
C *               REPUL                                              *
C *               PEGLEG                                             *
C *               OUTPEG                                             *
C *               LITER                                              *
C *               REDUCE                                             *
C *               PEGLEF                                             *
C *               FULCHM                                             *
C *               REDCHM                                             *
C *               OUTMAT                                             *
C *               MATOUT                                             *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE OUTPUT(H,U,MAD,C,E,W,IOCC,HDG,NDIM,NATOM,NTYPE,NC,
     .                  NHDG,FLAG,NEL)
C
C Matrix H  holds  the wavefunction coefficients.
C        C  holds  the overlap integrals.
C        E  holds  the eigenvalues.
C        U  will be overwritten and used for different purposes.
C
 
      INCLUDE 'iconpara.for'
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      DIMENSION H(NDIM,NDIM),U(NDIM,NDIM),MAD(NTYPE,NTYPE),C(NC),
     .          E(NDIM),W(NDIM),IOCC(NDIM),HDG(NHDG),NEL(9)
      REAL*8    MAD
      REAL*4    IOCC
      INTEGER   NEL
      LOGICAL*1 FLAG
C
C Variables that will be used for the MOBY-output
C
      INTEGER MAXANZ
      PARAMETER(MAXANZ=MXATOM*4)
      REAL*8  ECOPY(1:MAXANZ)
      REAL*8  UBCOPY(1:MXATOM)
 
      COMMON/ANEWDI/CWK,CWKALP,R(MXATOM,MXATOM)
 
      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
 
      COMMON /ATOMPT/IATMPT(4*MXATOM)

      COMMON/CNTRL/CON,PEEP,COULH,NH,NA,NATM,KA,NELEC,METH,IPRINT,
     .             IPUNCH,L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
      LOGICAL*1 L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
 
      COMMON/ENERGY/HSS(MXATOM),HPP(MXATOM),HDD(MXATOM)
 
      COMMON/FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .           NFRAG(9),NUMBAT(9),IFMOPT(4*MXATOM),IBCKPT(MXATOM),
     .           PRIFMO(7)
      INTEGER FMO,FMOCH,SORA,SORB
      LOGICAL*1 PRIFMO
  
      COMMON/ITPARM/DAMP1,DAMP2,DAMP3,LAMPRI,DELTAC,SENSE,MAXCYC,
     .              PRTCYC,ICYCLE,NCON,PARTIT,PRINTX,ITABLE(MXITER)
      REAL*8 LAMPRI
      LOGICAL*1 PARTIT,PRINTX,ITABLE
      INTEGER*4 PRTCYC
 
      COMMON/KSET/KATOM(MXSETK,2),SETK(MXSETK),SETD(MXSETK),NKSET
      INTEGER NKSET,KATOM
 
      COMMON /ORBRD/D0(4*MXATOM)

      COMMON /OUT/PRT(24),PUN(24),IOVPOP(24),IENRGY(24)
      INTEGER IOVPOP,IENRGY
      LOGICAL*1 PRT,PUN
 
      COMMON /REPBPA/EREPBP(MXATOM,MXATOM)

      COMMON /STARS/STAR,STAR2,SYMBOL(MXATOM)
      CHARACTER*2 STAR,STAR2,SYMBOL

      COMMON /TITLE/AB(10)
 
      LOGICAL*1   TMPONE
      CHARACTER*2 SYMB,HYDROG
      DATA HYDROG/' H'/
C
C 16.05.92 COPIES ENERGY VALUES BECAUSE THESE WILL BE
C          OVERWRITTEN LATER. ECOPY(I) WILL BE USED IN
C          THE MOBY ROUTINE./rr
C
      DO 10 I=1,NDIM
        ECOPY(I)=E(I)
   10 CONTINUE
 
      DO 20 I=1,NDIM
        NMIN=I
        IF (IOCC(I) .GT. 0.0001) GOTO 30
   20 CONTINUE
C
C  6.11.90/mbs
C
   30 IF (ITERAT .OR. FLAG) GOTO 210
C
C  CALCULATE AND PRINT OUT SUM OF ONE-ELECTRON ENERGIES.
C
      SUM=0.0D0
      DO 40 I=1,NDIM
        W(I)=DBLE(IOCC(I))
   40 SUM=SUM+E(I)*W(I)
      IF ( .NOT. PRT(9)) WRITE(6,50) SUM
   50 FORMAT(/,' SUM OF ONE-ELECTRON ENERGIES =',F16.8,' eV')
      IF (PUN(9)) WRITE(7,60) SUM
   60 FORMAT(F20.8)
C
C  CALCULATE AND PRINT OUT ORBITAL STABILIZATION ENERGY
C
      J=0
      DEORB=0.0D0
      IF (METH .GT. 0) THEN
C
C  The H(I,I) were stored in HSS(I),HPP(I) and HDD(I) in HUCKEL.
C  17.11.90/mbs
C
        DO 70 I=1,NATOM
          KEYI=KEY(I)
          IF (NS(KEYI) .NE. 0) THEN
            COUS=HSS(IBCKPT(I))
          ELSE
            COUS=0.0D0
          ENDIF
          IF (NP(KEYI) .NE. 0) THEN
            COUP=HPP(IBCKPT(I))
          ELSE
            COUP=0.0D0
          ENDIF
          IF (ND(KEYI) .NE. 0) THEN
            COUD=HDD(IBCKPT(I))
          ELSE
            COUD=0.0D0
          ENDIF
          CALL DESORB(COUD,COUP,COUS,I,IOCCS,IOCCP,IOCCD,KEYI)
          DEORB=DEORB+DBLE(IOCCS)*COUS+DBLE(IOCCP)*COUP+DBLE(IOCCD)*COUD
   70   CONTINUE
      ELSE
        DO 80 I=1,NATOM
          KEYI=KEY(I)
          COUD=COULD(KEYI)
          COUP=COULP(KEYI)
          COUS=COULS(KEYI)
          CALL DESORB(COUD,COUP,COUS,I,IOCCS,IOCCP,IOCCD,KEYI)
          DEORB=DEORB+DBLE(IOCCS)*COUS+DBLE(IOCCP)*COUP+DBLE(IOCCD)*COUD
   80   CONTINUE
      ENDIF
      DEORB=SUM-DEORB
      WRITE(6,90) DEORB
   90 FORMAT(' ORBITAL STABILIZATION ENERGY =',F16.8,' eV')
C
C  CALCULATE AND PRINT OUT CORE-CORE REPULSION ENERGY
C
      CALL REPUL(R,NATOM,EREP)
      WRITE(6,100) EREP
  100 FORMAT(' CORE-CORE REPULSION ENERGY   =',F16.8,' eV')
C
C  COMMENT IF CHARGE NOT EQUAL ZERO
C
      IF (KA .NE. 0) THEN
        WRITE(6,110)
  110   FORMAT(' BE CAREFUL TO INTERPRET REPULSION OF CHARGED ',
     .         'MOLECULES')
      ENDIF
C
C  PRINT OUT INTO FILE 'ENERTOT.DAT' THE FOLLOWING LINE:
C  TITLE; ENERGIES OF SUM OF 1-EL., ORB.STAB., CORE-CORE REPUL AND
C  STAB.+REPUL
C
      WRITE(2,120) AB,SUM,DEORB,EREP,EREP+DEORB
  120 FORMAT(8A8,A6,A2,4F16.8)
      EREP=EREP+DEORB
      WRITE(6,130) EREP
  130 FORMAT(' STABILIZATION + REPULSION    =',F16.8,' eV',/)
 
      IF (ONEMAT) GOTO 1080
C
C  PRINT OUT WAVE FUNCTIONS.
C
      IF (PRT(10)) GOTO 150
      WRITE(6,140)
  140 FORMAT('0WAVE FUNCTIONS',/,'0MO''S IN COLUMNS, AO''S IN ROWS')
      CALL OUTPEG(H,NDIM,NDIM)
  150 IF (PUN(10)) WRITE(7,160) H
  160 FORMAT(8F9.6)
C
C  IF DESIRED, PUNCH SECOND SECTION OF INPUT DECK FOR FMO PROGRAM.
C
C  Will not be used. 07.11.90/mbs
C
C      IF (IPUNCH.NE.10.AND.IPUNCH.NE.11) GOTO 500
C      WRITE(7,501) H
C 501   FORMAT(15A5)
C       IF (IPUNCH.NE.10) GOTO 500
C       IJ=1
C       U(1,1)=1.0D0
C       DO 503 I=2,NDIM
C         U(I,I)=1.0D0
C         I1=I-1
C         DO 503 J=1,I1
C           U(I,J)=C(IJ)
C           U(J,I)=C(IJ)
C 503       IJ=IJ+1
C       WRITE(7,501) U
C       WRITE(7,501) E
C 502   READ(5,*) NADD
C 504   FORMAT(I5)
C       IF (NADD.EQ.0) GOTO 500
C       DO 505 I=1,NADD
C         READ(5,*) AB
C 506     FORMAT(10A8)
C 505     WRITE(7,506) AB
C
C
C  CALCULATE AND PRINT OUT DENSITY MATRIX.
C
      IF (PRT(11) .AND. .NOT. PUN(11)) GOTO 210
      DO 180 I=1,NDIM
        DO 180 J=1,I
          U(I,J)=0.0D0
          DO 170 K=1,NDIM
  170     U(I,J)=U(I,J)+H(I,K)*H(J,K)*W(K)
  180 U(J,I)=U(I,J)
      IF (PRT(11)) GOTO 200
      WRITE(6,190)
  190 FORMAT('0DENSITY MATRIX')
      CALL OUTPEG(U,NDIM,NDIM)
  200 IF (PUN(11)) WRITE(7,160) U
C
C
C  CALCULATE ATOMIC ORBITAL OCCUPATIONS AND STORE IN E(I).
C  CALCULATE OVERLAP POPULATION MATRIX.
C
  210 IJ=1
      DO 240 I=1,NDIM
        E(I)=0.0D0
        DO 240 J=1,I
          UB=0.0D0
          DO 220 K=NMIN,NDIM
  220     UB=UB+H(I,K)*H(J,K)*DBLE(IOCC(K))
          UB=UB*0.5D0
          IF (I .EQ. J) GOTO 230
          UB=(UB+UB)*C(IJ)
          IJ=IJ+1
  230     E(I)=E(I)+UB
          E(J)=E(J)+UB
          IF (ITERAT) GOTO 240
          UB=UB+UB
          U(I,J)=UB
          U(J,I)=UB
  240 CONTINUE
C
C  IF DOING CHARGE ITERATION ( METH=1 ) CALL LITER.
C  X,Y,Z changed to HSS,HPP,HDD. 17.11.90/mbs
C
      IF (.NOT. ITERAT) GOTO 320
      K=ICYCLE
      PRINTX=((ICYCLE/PRTCYC)*PRTCYC .EQ. ICYCLE)
      IF (ICYCLE .EQ. MAXCYC) PRINTX=.TRUE.
      CALL LITER(NH,NA,E,W,J)
      IF (ICYCLE .EQ. 15000) PRINTX=.TRUE.
      IF (K .EQ. 1) WRITE(6,250)
  250 FORMAT('1     ATOMIC CHARGES',//)
      GOTO (260,280,290,300) ,J
  260 WRITE(6,270) K,(HSS(I),I=1,NATOM)
  270 FORMAT(/,T3,'CYCLE NO.',I3,(T20,10F10.5))
      GOTO 310
  280 WRITE(6,270) K,(HPP(I),I=1,NATOM)
      GOTO 310
  290 WRITE(6,270) K,(HDD(I),I=1,NATOM)
      GOTO 310
  300 WRITE(6,270) K,(W(I),I=1,NATOM)
  310 RETURN
C
C
C  Print only some orbitals for wavefunctions in terms of FMO.
C  07.11.90/mbs
C
C  PRINT OUT OVERLAP POPULATION MATRIX.
C
  320 IF (PRT(12)) GOTO 340
      WRITE(6,330) NELEC
  330 FORMAT('0OVERLAP POPULATION MATRIX FOR',I4,' ELECTRONS')
      IF (.NOT. FLAG) CALL OUTPEG(U,NDIM,NDIM)
      IF (FLAG) CALL MATOUT(U,NDIM,NDIM)
  340 IF (PUN(12)) WRITE(7,160) U
C
C  CALL REDUCE TO CALCULATE REDUCED OVERLAP MATRIX. PRINT OUT
C  REDUCED OVERLAP MATRIX.
C
C  07.11.90/mbs
C
      IF (PRT(13) .AND.  .NOT. PUN(13)) GOTO 390
      CALL REDUCE(U,NDIM,NATOM)
      DO 350 I=2,NATOM
        K=I-1
        DO 350 J=1,K
  350 U(J,I)=U(I,J)
      IF (PRT(13)) GOTO 380
      IF (.NOT. FLAG) WRITE(6,360)
  360 FORMAT('0REDUCED OVERLAP POPULATION MATRIX, ATOM BY ATOM')
      IF (FLAG) WRITE(6,370)
  370 FORMAT('0REDUCED OVERLAP POPULATION MATRIX, FRAGMENT BY ',
     .       'FRAGMENT')
      IF (.NOT. FLAG) CALL PEGLEG(U,NATOM,NDIM)
      IF (FLAG) CALL PEGLEF(U,NATOM,NDIM,NATOM)
  380 IF (PUN(13)) WRITE(7,160) ((U(I,J),I=1,NATOM),J=1,NATOM)
C
C  IF L3 IS TRUE, CALCULATE AND PRINT OUT OVERLAP POPULATION ANALYSIS,
C  ORBITAL BY ORBITAL, FOR EACH MOLECULAR ORBITAL SPECIFIED.
C
  390 IF (.NOT. L3) GOTO 480
      DO 450 N=1,23,2
        IF (IOVPOP(N) .EQ. 0) GOTO 460
        KMIN=IOVPOP(N)
        KMAX=IOVPOP(N+1)
        DO 450 K=KMIN,KMAX
          WRITE(6,400) K,W(K)
  400     FORMAT(/,'0OVERLAP POPULATION MATRIX, ORBITAL BY ORBITAL, ',
     .           'FOR MOLECULAR ORBITAL',I4,5X,'OCCUPATION IS',F7.4,
     .           ' ELECTRONS')
          IF (IOCC(K) .LT. 0.0001) GOTO 450
          SUM=0.5D0*W(K)
          IJ=1
          DO 420 I=1,NDIM
            DO 420 J=1,I
              UB=H(I,K)*H(J,K)
              IF (I .EQ. J) GOTO 410
              UB=(UB+UB)*C(IJ)
              IJ=IJ+1
  410         UB=(UB+UB)*SUM
              U(J,I)=UB
              U(I,J)=UB
  420     CONTINUE
C
C  07.11.90/mbs, adapted from BIGICON
C
C          IF (.NOT.FLAG) CALL OUTPEG(U,NDIM,NDIM)
C          IF (FLAG) CALL MATOUT(U,NDIM,NDIM)
C
C  Reduced overlap matrix for orbital by orbital.
C
          CALL REDUCE(U,NDIM,NATOM)
          DO 430 I=2,NATOM
            KK=I-1
            DO 430 J=1,KK
  430     U(J,I)=U(I,J)
          WRITE(6,440) K
  440     FORMAT(/,'0REDUCED OVERLAP POPULATION MATRIX, ATOM BY ATOM, ',
     .           'FOR MOLECULAR ORBITAL',I4,/)
          IF (.NOT. FLAG) CALL PEGLEG(U,NATOM,NDIM)
          IF (FLAG) CALL PEGLEF(U,NATOM,NDIM,NATOM)
  450 CONTINUE
  460 WRITE(6,470)
  470 FORMAT(///)
C
C  CALL FULCHM TO CALCULATE COMPLETE CHARGE MATRIX. PRINT OUT
C  COMPLETE CHARGE MATRIX.
C
  480 L1=PRT(14) .AND. .NOT. PUN(14)
      L2=PRT(15) .AND. .NOT. PUN(15)
      IF (L1 .AND. L2) GOTO 540
      CALL FULCHM(W,U,C,H,NDIM)
      IF (PRT(14)) GOTO 500
      WRITE(6,490)
  490 FORMAT('0COMPLETE CHARGE MATRIX FOR EACH MO, NORMALIZED TO TWO ',
     .       'ELECTRONS REGARDLESS OF OCCUPATION')
      IF (.NOT. FLAG) CALL OUTPEG(U,NDIM,NDIM)
      IF (FLAG) CALL MATOUT(U,NDIM,NDIM)
  500 IF (PUN(14)) WRITE(7,160) U
C
C  CALL REDCHM TO CALCULATE REDUCED CHARGE MATRIX. PRINT OUT
C  REDUCED CHARGE MATRIX.
C
      IF (L2) GOTO 540
      CALL REDCHM(U,NDIM,NATOM)
      IF (PRT(15)) GOTO 530
      IF (.NOT. FLAG) WRITE(6,510)
  510 FORMAT('0REDUCED CHARGE MATRIX, MO''S IN COLUMNS, ATOMS IN ',
     .       'ROWS, NORMALIZED TO TWO ELECTRONS REGARDLESS OF ',
     .       'OCCUPATION')
      IF (FLAG) WRITE(6,520)
  520 FORMAT('0REDUCED CHARGE MATRIX, MO''S IN COLUMNS, FRAGMENTS IN ',
     .       'ROWS')
      IF (.NOT. FLAG) CALL OUTMAT(U,NDIM,NATOM,NDIM)
      IF (FLAG) CALL PEGLEF(U,NATOM,NDIM,NDIM)
  530 IF (PUN(15)) WRITE(7,160) ((U(I,J),I=1,NATOM),J=1,NDIM)
C
C  PRINT OUT ATOMIC CHARGES AND ORBITAL OCCUPATIONS.
C  Dieser Teil muss sehr wahrscheinlich einmal abgeaendert werden, da
C  das artefaktische NH sowieso nicht mehr verwendet wird. 07.11.90/mbs
C
  540 IF (PRT(16)) GOTO 660
      IF (.NOT. FLAG) THEN
        WRITE(6,550)
  550   FORMAT('0ATOM',T12,'NET CHG.',T35,'ATOMIC ORBITAL OCCUPATION ',
     .         'FOR GIVEN MO OCCUPATION',/,T35,'S',T45,'X',T55,'Y',T65,
     .         'Z',T75,'X2-Y2',T85,'Z2',T95,'XY',T105,'XZ',T115,'YZ',/)
        SYMB=HYDROG
        J=1
        DO 590 I=1,NATOM
          IF (I .GT. NH) GOTO 560
          UB=1.0D0-E(I)
          N=I
          GOTO 570
  560     KITE=I-NH
          KEYI=KEY(KITE)
          SYMB=SYMBOL(KEYI)
          UB=VELEC(KEYI)-E(J)
          N=J
          IF (NP(KEYI) .EQ. 0) GOTO 570
          UB=UB-E(J+1)-E(J+2)-E(J+3)
          N=J+3
          IF (ND(KEYI) .EQ. 0) GOTO 570
          UB=UB-E(J+4)-E(J+5)-E(J+6)-E(J+7)-E(J+8)
          N=J+8
  570     WRITE(6,580) SYMB,I,UB,(E(K),K=J,N)
  580     FORMAT(1X,A2,I3,T10,F10.5,T30,9F10.5)
C
C  15.05.92 CHARGES WILL BE COPIED FROM UB to UBCOPY. UBCOPY
C           WILL BE USED IN THE MOBY ROUTINE. /rr
C
          UBCOPY(I)=UB
  590   J=N+1
      ELSE
C
C  07.11.90/mbs, 27.11.90/mbs
C  Fragment charges.
C
        WRITE(6,600)
  600   FORMAT('0ATOM',T18,'NET CHG.',T38,'FRAGMENT MO OCCUPATION ',
     .         'FOR GIVEN MOLECULAR MO OCCUPATION',//)
        IS=1
        DO 650 I=1,NATOM
          UB=NEL(I)
          IE=SORB(I+1)-1
          DO 610 J=IS,IE
  610     UB=UB-E(J)
          KITE=IS-1
  620     LOW=KITE+1
          KITE=KITE+8
          IF (KITE .GT. IE) KITE=IE
          IF ((KITE-IS) .LE. 8) WRITE(6,630) I,UB,(E(J),J=LOW,KITE)
          IF ((KITE-IS) .GT. 8) WRITE(6,640) (E(J),J=LOW,KITE)
  630     FORMAT(' FRAGMENT ',I1,':',2X,F10.5,T30,8F10.5)
  640     FORMAT(T30,8F10.5)
          IF (KITE .LT. IE) GOTO 620
  650   IS=IE+1
      ENDIF
C
C 15.05.92/rr
C CAUSES OUTPUT IN MOBY FORMAT
C
      IF (PRT(22)) THEN
        CALL MOBY(ECOPY,H,NDIM,X,Y,Z,NA,SYMBOL,KEY,UBCOPY,R,SORB)
        PRT(22)=.FALSE.
      ENDIF
C
C  IF CALCULATING ENERGY MATRIX, PUT DIAGONAL ELEMENTS OF HUCKEL
C  MATRIX IN W(I).
C  X,Y,Z changed to HSS,HPP,HDD. 17.11.90/mbs
C
  660 IF (FLAG) RETURN
 
      PRT(2)=PRT(18) .AND. .NOT. PUN(18)
      PRT(4)=PRT(20) .AND. .NOT. PUN(20)
      L1=(PRT(17) .AND. .NOT. PUN(17)) .AND. (PRT(2))
      L2=(PRT(19) .AND. .NOT. PUN(19)) .AND. (PRT(4))
      IF (L1 .AND. L2 .AND. .NOT. L4) GOTO 1080
 
      IF (NH .EQ. 0) GOTO 680
      DO 670 I=1,NH
  670 W(I)=HSS(IBCKPT(I))
  680 J=NH+1
      K=NH+1
      DO 690 I=1,NA
        KEYI=KEY(I)
        W(J)=HSS(IBCKPT(K))
        J=J+1
        IF (NP(KEYI) .EQ. 0) GOTO 690
        UB=HPP(IBCKPT(K))
        W(J)=UB
        W(J+1)=UB
        W(J+2)=UB
        J=J+3
        IF (ND(KEYI) .EQ. 0) GOTO 690
        UB=HDD(IBCKPT(K))
        W(J)=UB
        W(J+1)=UB
        W(J+2)=UB
        W(J+3)=UB
        W(J+4)=UB
        J=J+5
  690 K=K+1
C
C  IF DOING CHARGE ITERATION WITH MADELUNG CORRECTION ON MOLECULE
C  WITH NON-ZERO CHARGE, PUT MADELUNG TERMS IN E(I).
C
      QON=DBLE(KA)/DBLE(NELEC)
      IF (METH .LT. 3 .OR. DABS(QON) .LT. 0.0001D0) GOTO 730
      IF (NH .EQ. 0) GOTO 710
      DO 700 I=1,NH
  700 E(I)=MAD(I,I)
  710 J=NH+1
      K=NH+1
      DO 720 I=1,NA
        KEYI=KEY(I)
        E(J)=MAD(K,K)
        J=J+1
        K=K+1
        IF (NP(KEYI) .EQ. 0) GOTO 720
        UB=MAD(K,K)
        E(J)=UB
        E(J+1)=UB
        E(J+2)=UB
        J=J+3
        K=K+1
        IF (ND(KEYI) .EQ. 0) GOTO 720
        UB=MAD(K,K)
        E(J)=UB
        E(J+1)=UB
        E(J+2)=UB
        E(J+3)=UB
        E(J+4)=UB
        J=J+5
        K=K+1
  720 CONTINUE
C
C  CALCULATE AND PRINT OUT ENERGY MATRIX.
C  19.11.90/mbs, 01.06.92/mbs
C
  730 TMPONE=ONEMAT
      ONEMAT=.TRUE.
      IF (L1) GOTO 970
  740 SUM=1.0D0
      IF (ONEMAT) SUM=0.0D0
      IJ=1
      CNST=2.0D0*CON
      CONST=CNST
      DO 800 I=1,NDIM
        U(I,I)=0.0D0
        IAOI=IFMOPT(I)
        IATOM1=IATMPT(IAOI)
        DO 800 J=1,I
          IAOJ=IFMOPT(J)
          IATOM2=IATMPT(IAOJ)
          CDELTA=CWKALP
          CN2=CONST
          UB=0.0D0
          DO 750 K=NMIN,NDIM
  750     UB=UB+H(I,K)*H(J,K)*DBLE(IOCC(K))
          IF (I .EQ. J) GOTO 800
          IF ((.NOT. L5) .AND. (IPUNCH .EQ. 0)) GOTO 770
          IF ((IPUNCH .NE. 0) .AND. L7) THEN
            DO 755 IATOM=1,NKSET
              IF (((IATOM1 .EQ. KATOM(IATOM,1)) .AND.
     .             (IATOM2 .EQ. KATOM(IATOM,2))) .OR.
     .            ((IATOM1 .EQ. KATOM(IATOM,2)) .AND.
     .             (IATOM2 .EQ. KATOM(IATOM,1)))) THEN
                CN2=SETK(IATOM)
                IF (SETD(IATOM) .GT. 1.D-3) THEN
                  CDELTA=SETD(IATOM)
                ENDIF
                GOTO 757
              ENDIF
  755       CONTINUE
          ENDIF
  757     UC=W(I)
          ET=W(J)
          IF (NHDG .EQ. 1) GOTO 760
          UC=HDG(I)
          ET=HDG(J)
  760     UC=(UC-ET)/(UC+ET)
          UC=UC*UC
          CNST=CN2+UC+UC*UC*(1.0D0-CN2)
          IF (IPUNCH .NE. 0) THEN
            ORBRD=D0(IAOI)+D0(IAOJ)
            ABDIST=R(IATOM1,IATOM2)
            RDIF=ABDIST-ORBRD
            RD2=(RDIF-DABS(RDIF))*CDELTA           
            CNST=1.0D0+(CNST-1.0D0)*DEXP(-CDELTA*RDIF)/(1.0D0+RD2*RD2)
          ENDIF
  770     IF ((METH .LT. 3) .OR. (DABS(QON) .LT. 0.0001D0)) THEN
            UC=UB*(CNST-SUM)*C(IJ)*(W(I)+W(J))
          ELSE
            UC=UB*C(IJ)*((CNST-SUM)*(W(I)+W(J))-QON*(1.0D0-CNST)*(E(I)+
     .         E(J)))
          ENDIF
          UB=SUM*UB*C(IJ)
          IJ=IJ+1
          U(J,I)=UC
          U(I,J)=UC
          U(J,J)=U(J,J)+UB*W(J)
  800 U(I,I)=U(I,I)+UB*W(I)
      IF (PRT(17)) GOTO 830
      IF (ONEMAT) WRITE(6,810)
  810 FORMAT(/,'0ENERGY MATRIX')
      IF ( .NOT. ONEMAT) WRITE(6,820)
  820 FORMAT('0ENERGY PARTITIONING')
C
C 07.11.90/mbs. For future applications. See label 660.
C
      IF ( .NOT. FLAG) CALL OUTPEG(U,NDIM,NDIM)
      IF (FLAG) CALL MATOUT(U,NDIM,NDIM)
  830 IF (PUN(17)) WRITE(7,840) U
  840 FORMAT(8F9.5)
C
C  CALL REDUCE TO CALCULATE REDUCED ENERGY MATRIX. PRINT OUT
C  REDUCED ENERGY MATRIX.
C
      IF (PRT(2)) GOTO 970
      CALL REDUCE(U,NDIM,NATOM)
      DO 850 I=2,NATOM
        K=I-1
        DO 850 J=1,K
  850 U(J,I)=U(I,J)
      IF (PRT(18)) GOTO 950
C
C 07.11.90/mbs. For future applications. See label 660.
C
      IF (ONEMAT .AND.  .NOT. FLAG) WRITE(6,860)
  860 FORMAT('0REDUCED ENERGY MATRIX, ATOM BY ATOM')
      IF (ONEMAT .AND. FLAG) WRITE(6,870)
  870 FORMAT('0REDUCED ENERGY MATRIX, FRAGMENT BY FRAGMENT')
      IF ( .NOT. ONEMAT .AND.  .NOT. FLAG) WRITE(6,880)
  880 FORMAT('0REDUCED ENERGY PARTITIONING, ATOM BY ATOM')
      IF ( .NOT. ONEMAT .AND. FLAG) WRITE(6,890)
  890 FORMAT('0REDUCED ENERGY PARTITIONING, FRAGMENT BY FRAGMENT')
 
      KITE=0
  900 LOW=KITE+1
      KITE=KITE+13
      IF (KITE .GT. NATOM) KITE=NATOM
      WRITE(6,910) (I,I=LOW,KITE)
  910 FORMAT(/5X,13I9,/)
      WRITE(6,920)
  920 FORMAT()
      DO 930 I=1,NATOM
  930 WRITE(6,940) I,(U(I,J),J=LOW,KITE)
  940 FORMAT(I5,2X,13F9.4)
      IF (KITE .LT. NATOM) GOTO 900
  950 IF (PUN(18)) WRITE(7,960) ((U(I,J),I=1,NATOM),J=1,NATOM)
  960 FORMAT(7F10.5)
C
C  IF L4 IS TRUE, CALCULATE AND PRINT OUT ENERGY MATRIX ANALYSIS,
C  ORBITAL BY ORBITAL, FOR EACH MOLECULAR ORBITAL SPECIFIED.
C  Distance dependent weighted Wolfsberg-Helmholz formula is not
C  incorporated, because this option is not used in ICONC. If wanted,
C  it can be programmed according to subroutine NEWDIA. 01.06.92/mbs
C
  970 IF (.NOT. ONEMAT) GOTO 1080
      IF (.NOT. L4) GOTO 1070
      DO 1050 N=1,23,2
        IF (IENRGY(N) .EQ. 0) GOTO 1060
        KMIN=IENRGY(N)
        KMAX=IENRGY(N+1)
        DO 1050 K=KMIN,KMAX
          WRITE(6,980) K,IOCC(K)
  980     FORMAT(/,'0ENERGY MATRIX, ORBITAL BY ORBITAL, FOR MOLECULAR ',
     .           'ORBITAL',I4,5X,'OCCUPATION IS',F7.4,' ELECTRONS')
          IF (IOCC(K) .LT. 0.0001) GOTO 1050
          IJ=1
          CNST=CON
          EX=DBLE(IOCC(K))
          DO 1040 I=1,NDIM
            DO 1040 J=1,I
              UB=H(I,K)*H(J,K)
              IF (I .EQ. J) GOTO 1030
              IF (.NOT. L5) GOTO 1000
              UC=W(I)
              ET=W(J)
              IF (NHDG .EQ. 1) GOTO 990
              UC=HDG(I)
              ET=HDG(J)
  990         UC=(UC-ET)/(UC+ET)
              UC=UC*UC
              CNST=CON+UC/2.0D0+UC*UC*(0.5D0-CON)
 1000         IF (METH .LT. 3 .OR. DABS(QON) .LT. 0.0001D0) GOTO 1010
              UB=UB*C(IJ)*(CNST*(W(I)+W(J))-QON*(0.5D0-CNST)*(E(I)+
     .              E(J)))
              GOTO 1020
 1010         UB=UB*CNST*C(IJ)*(W(I)+W(J))
 1020         IJ=IJ+1
              UB=2.0D0*UB*EX
              U(I,J)=UB
              U(J,I)=UB
              GOTO 1040
 1030         U(I,I)=UB*W(I)*EX
 1040     CONTINUE
C
C 07.11.90/mbs. For future applications. See label 660.
C
          IF ( .NOT. FLAG) CALL OUTPEG(U,NDIM,NDIM)
          IF (FLAG) CALL MATOUT(U,NDIM,NDIM)
 1050 CONTINUE
 1060 WRITE(6,470)
C
C  CALCULATE AND PRINT OUT ENERGY PARTITIONING AND REDUCED
C  ENERGY PARTITIONING.
C
 1070 IF (L2) GOTO 1080
      ONEMAT=.FALSE.
      PRT(17)=PRT(19)
      PUN(17)=PUN(19)
      PRT(18)=PRT(20)
      PUN(18)=PUN(20)
      PRT(2)=PRT(4)
      GOTO 740
C
C 19.11.90/mbs
C
 1080 ONEMAT=TMPONE
      IF (.NOT. PRT(21)) THEN
        WRITE(6,1090)
 1090   FORMAT(/,'0REPULSION ENERGY BETWEEN ATOM PAIRS')
        CALL PEGLEG(EREPBP,NATOM,MXATOM)
      ENDIF
      RETURN
      END
