C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE QRTN                                                 *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Performs a QR transform on a real symmetric tri-   *
C *               diagonal matrix.                                   *
C *                                                                  *
C *               N IS THE DIMENSION OF THE MATRIX. A CONTAINS THE N *
C *               DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX. B     *
C *               CONTAINS THE N-1 OFF-DIAGONAL ELEMENTS. M IS THE   *
C *               MAXIMUM NUMBER OF ITERATIONS (SAY 20). NJX IS THE  *
C *               PHYSICAL ROW DIMENSION OF THE EIGENVECTOR MATRIX V.*
C *                                                                  *
C *               THE EIGENVALUES ARE ASSUMED KNOWN AND PLACED IN    *
C *               THE FIRST N ELEMENTS OF EIG.                       *
C *                                                                  *
C *               N VECTORS V (EACH OF LENGTH N) ARE TRANSFORMED IN- *
C *               TO THE BASIS IN WHICH THE MATRIX IS DIAGONAL.      *
C *                                                                  *
C *  Authors....: see subroutine GIVENS                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *  Date.......: -                                                  *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: GIVENS                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE QRTN (A,B,V,EIG,N,M,TOL,NJX)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1),V(1),EIG(1)

      NX = N
      NNM1 = NJX*(NX-1)
      NI = 1
C
C  SET INITIAL TOTAL SHIFT.
C
      SH = 0
      IF (NX-2) 50,60,1
C
C  K COUNTS THE NUMBER OF ITERATIONS PER EIGENVALUE.
C
1     K=0
C
C  SET INITIAL TEMPORARY SHIFT.
C
98    ST = EIG(NX)-SH
C
C  CHECK FOR SMALL OFF-DIAGONAL ELEMENTS.
C
      IT = NX
99    IT = IT-1
      IF (DABS(B(IT)).LE.TOL) GOTO 40
      IF (IT.GT.NI) GOTO 99
C
C  NO SMALL OFF-DIAGONAL ELEMENTS FOUND.  ITERATE.  EACH NEW
C  ITERATION BEGINS HERE.
C
100   K = K+1
      IF (K.LT.M) GOTO 11
      WRITE (6,1000) K
1000  FORMAT(35H NO CONVERGENCE OF QR ALGORITHM IN   ,I4,11H ITERATIONS)
      STOP
11    IF (K.EQ.1) GOTO 15
C
C  SOLVE THE TWO BY TWO IN THE LOWER RIGHT CORNER AND USE SMALLER ROOT
C  AS THE NEXT SHIFT.
C
12    AT = A(NX)+A(NX-1)
      ST = AT*5.D-1
      DISC = AT**2-4.D0*(A(NX)*A(NX-1)-B(NX-1)**2)
      IF (DISC.LE.0.D0) GOTO 15
      ST = ST-DSIGN(DSQRT(DISC),ST)*5.D-1
C
C  INCREASE THE TOTAL SHIFT BY THE TEMPORARY SHIFT.
C
15    SH = SH+ST
C
C  THIS LOOP SUBTRACTS THE TEMPORARY SHIFT FROM THE DIAGONAL ELEMENTS.
C
      DO 20 I=1,NX
20      A(I) = A(I)-ST
      R = DSQRT(A(NI)**2+B(NI)**2)
      S = B(NI)/R
      CS = S
      C = A(NI)/R
      U = (S**2)*(A(NI)+A(NI+1))
      A(NI) = A(NI)+U
      CALL ROTATE(V(NI),C,S,NJX,NNM1)
      NTOP = NX-2
C
C  THIS LOOP COMPLETES ONE ITERATION, THAT IS ONE QR TRANSFORM.
C
      DO 10 I=NI,NTOP
C
C  G IS THE GAMMA AND Q IS THE P IN THE NOTATION OF WILKINSON.
C
        G = A(I+1)-U
        Q = C*A(I+1)-CS*B(I)
        R = DSQRT(Q**2+B(I+1)**2)
C
C  ROTATE AN OFF-DIAGONAL ELEMENT.
C
        B(I) = S*R
C
C  FIND THE NEW SINE AND COSINE FOR THE JACOBI ROTATION. THEN
C  COMPUTE A NEW U.
C
        S = B(I+1)/R
        CS = C*S
        C = Q/R
        U = (S**2)*(G+A(I+2))
C
C  ROTATE A DIAGONAL ELEMENT.
C
        A(I+1) = G+U
C
C  ROTATE THE VECTORS.
C
        CALL ROTATE (V(I+1),C,S,NJX,NNM1)
10    CONTINUE
C
C  COMPUTE THE LAST OFF DIAGONAL ELEMENT.
C
      B(NTOP+1) = S*(C*A(NX)-CS*B(NTOP+1))
C
C  COMPUTE THE LAST DIAGONAL ELEMENT.
C
      A(NX) = A(NX)-U
C
C  END OF ONE ITERATION.
C
85    IT = NX
C
C  CHECK UPWARD THROUGH THE OFF DIAGONAL ELEMENTS TO FIND THOSE LESS
C  THAN TOL. IF NO OFF-DIAGONAL ELEMENTS LESS THAN TOL ARE FOUND,
C  PERFORM ANOTHER ITERATION.
C
30    IT = IT-1
      IF (DABS(B(IT)).LE.TOL) GOTO 40
      IF (IT-NI) 100,100,30
C
C  BRANCH ACCORDING TO WHETHER THE MATRIX ISOLATED BY THE SMALL
C  OFF-DIAGONAL ELEMENT IS OF DIMENSION ONE, TWO, OR MORE.
C
40    IF (NX-IT-2) 50,60,70
C
C  EXTRACT THE EIGENVALUE OF A ONE BY ONE MATRIX BY ADDING BACK
C  THE SHIFT.
C
50    A(NX) = A(NX)+SH
C
C  DECREASE THE SIZE OF THE PORTION OF THE MATRIX AFFECTED BY
C  LATER ITERATIONS.
C
      NX = NX-1
C
C  RESET THE ITERATION COUNTER.
C
      K = 0
      GOTO 80
C
C  EXTRACT THE EIGENVALUES FROM A TWO BY TWO MATRIX AND PERFORM THE
C  CORRESPONDING ROTATIONS ON THE VECTORS.
C
60    AL = -B(NX-1)
      AM = 5.D-1*(A(NX-1)-A(NX))
      AN = DSQRT(AL**2+AM**2)
      C = DSQRT((AN+DABS(AM))/(2.D0*AN))
      S = DSIGN(5.D-1,AM)*AL/(AN*C)
      TA = A(NX-1)
      TB = A(NX)
      TC = B(NX-1)
      CX = C**2
      SX = S**2
      CS = C*S
C
C  ROTATE THE DIAGONAL ELEMENTS, THE OFF-DIAGONAL ELEMENTS, AND
C  THE VECTORS.
C
      A(NX-1) = TA*CX+TB*SX-2.D0*TC*CS+SH
      A(NX) = TA*SX+TB*CX+2.D0*TC*CS+SH
      B(NX-1) = 2.D0*AM*CS+TC*(CX-SX)
      I = NX-1
      S = -S
      CALL ROTATE (V(I),C,S,NJX,NNM1)
C
C  RESET THE ITERATION COUNTER.
C
      K = 0
C
C  DECREASE THE SIZE OF THE PORTION OF THE MATRIX AFFECTED BY THE
C  LATER ITERATIONS.
C
      NX = NX-2
      GOTO 80
C
C  THE NEXT STATEMENT IS REACHED WHEN THE PORTION OF THE MATRIX
C  ISOLATED IS GREATER THAN TWO BY TWO.  IT CHANGES THE LOWER LIMIT
C  OF THE ITERATION SO THAT ONLY THIS PORTION WILL BE AFFECTED BY
C  SUBSEQUENT ROTATIONS UNTIL ALL ITS EIGENVALUES ARE FOUND.
C
70    NI = IT+1
C
C  TRANSFER TO BEGINNING OF ANOTHER ITERATION.
C
      GOTO 100
C
C  NEXT STATEMENT IS REACHED AFTER EITHER ONE OR TWO EIGENVALUES
C  HAVE JUST BEEN FOUND.  IT TRANSFERS IF ALL THE EIGENVALUES IN
C  THIS PORTION OF THE MATRIX HAVE BEEN FOUND.
C
80    IF (NX.LT.NI) GOTO 90
C
C  BRANCH ACCORDING TO WHETHER ONE, TWO, OR MORE EIGENVALUES REMAIN
C  TO BE FOUND.
C
95    IF (NX-NI-1) 50,60,98
C
C  THE NEXT STATEMENT IS REACHED WHEN ALL EIGENVALUES IN THIS PART
C  OF THE MATRIX HAVE BEEN FOUND. IT RETURNS IF THIS IS THE LAST
C  PART OF THE MATRIX.
C
90    IF (NI.EQ.1) RETURN
C
C  ENLARGE THE PORTION OF THE MATRIX BEING TREATED TO INCLUDE THE
C  BEGINNING OF THE MATRIX.
C
      NI = 1
      GOTO 95
      END
