C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE REPUL                                                *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Calculates and returns core-core-repulsion energy  *
C *               EREP.                                              *
C *                                                                  *
C *  Authors....: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: last revision 05.05.93/mbs (Martin Braendle)       *
C *  Remarks....: Omitting-atoms-from-repulsion-energy-calculation   *
C *               option added.                                      *
C *               Repulsion energies between atom pairs will be sto- *
C *               red for output.                                    *
C *                                                                  *
C *  Called from: OUTPUT                                             *
C *                                                                  *
C *  Calls......: REP                                                *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE REPUL(R,NATOM,EREP)
 
      INCLUDE 'iconpara.for'
 
      IMPLICIT REAL*8 (A-H,O-Z)
 
      DIMENSION R(MXATOM,MXATOM)
 
      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
 
      COMMON/CNTRL/CON,PEEP,COULH,NH,NA,NATM,KA,NELEC,METH,IPRINT,
     .             IPUNCH,L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
      LOGICAL*1 L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT

      COMMON/FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .           NFRAG(9),NUMBAT(9),IFMOPT(4*MXATOM),IBCKPT(MXATOM),
     .           PRIFMO(7)
      INTEGER FMO,FMOCH,SORA,SORB
      LOGICAL*1 PRIFMO
   
      COMMON/OMITRP/NOMIT,OMIT(MXOMIT)
      INTEGER NOMIT,OMIT

      COMMON /REPBPA/EREPBP(MXATOM,MXATOM)

      DO 50 I=1,NATOM
        EREPBP(I,I)=0.D0
50    CONTINUE
 
      EREP=0.
      DO 10 I=1,(NATOM-1)
        IF (L8.AND.(I.EQ.1)) THEN
          DO 30 IATOM=1,NOMIT
            IF (OMIT(IATOM).EQ.1) GOTO 10
30        CONTINUE
        ENDIF
 
        KEYI=KEY(I)
        DO 20 J=(I+1),NATOM
          IF (L8) THEN
            DO 40 IATOM=1,NOMIT
              IF (OMIT(IATOM).EQ.J) GOTO 20
40          CONTINUE
          ENDIF
          IATOM1=IBCKPT(I)
          IATOM2=IBCKPT(J)
          RX=R(IATOM1,IATOM2)
          KEYJ=KEY(J)
          ZA=VELEC(KEYI)
          ZB=VELEC(KEYJ)
          CALL REP(RX,EREPA,IATOM1,KEYI)
          CALL REP(RX,EREPB,IATOM2,KEYJ)
          EREP1=ZA*ZB/RX-((ZA*EREPB+ZB*EREPA)/2.0)
          EREP1=EREP1*(14.395)
          EREPBP(I,J)=EREP1
          EREPBP(J,I)=EREP1
          EREP=EREP+EREP1
20      CONTINUE
10    CONTINUE
      RETURN
      END
